unit FigFileMaker;

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QGraphics, IniFiles, Libc, QDialogs;
{$else}
  Windows, Messages, SysUtils, Classes, Graphics, IniFiles, Registry, Forms, Dialogs;
{$endif}

type
  TXYGrapher = class

private
public
  NumLegend, Height, Width, Lmargin, Tmargin : Integer;
  NumFontSize, NumDataSeries :ShortInt;
  Xb, Xt, Yl, Yr, ReadStart, ReadEnd : Array [1..10] of Integer;
  LineStyle, LineColor, MarkStyle, MarkColor : Array [1..10] of Integer;
  LineWidth, MarkSize : Array [1..10] of Double;
  Xmult, Xoffset, Ymult, Yoffset : Array [1..10] of Double;
  DataSeries : Array [1..10] of String;
  LegendLspacing, LegendXshift, LegendYshift : Array [1..10] of Single;
  Max, Min : Array[1..4] of Double;
  Division, Minordivision, logarithm, ZeroLine : Array[1..4] of ShortInt;
  NumeralsXshift, NumeralsYshift : Array[1..4] of Double;
  Legend : Array [1..10] of String;
  FLineWidth: Double;
  NumFontName : TFontName;
  MyDocDir, IniFileName: String;
  constructor Create;
  procedure SaveFigFile(FileName: String);
  procedure ExecuteXYGrapher(FileName: String);
  procedure SetFormSize(Width, Height: Integer);
end;

implementation

constructor TXYGrapher.Create;
{$ifdef MSWINDOWS}
var
  Reg: TRegistry;
  S: String;
{$endif}
begin
  NumDataSeries := 0;
  DataSeries[1] := '';
  Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 1; ReadEnd[1] := 0;
  LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 33023; MarkStyle[1] := 0; MarkSize[1] := 0;
  Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
  //----------
  Height := 65; Width := 65; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
  NumFontName := 'Arial';
  Max[1] := 1; Min[1] := 0; Division[1] := 5; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
  Max[2] := 1; Min[2] := 0; Division[2] := 5; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
  Max[3] := 1; Min[3] := 0; Division[3] := 5; Minordivision[3] := 2; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
  Max[4] := 1; Min[4] := 0; Division[4] := 5; Minordivision[4] := 2; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
  NumLegend := 5;
  Legend[1] := 'Xb-axis\n';
  Legend[2] := '';
  Legend[3] := '';
  Legend[4] := 'Yl-axis\n';
  Legend[5] := '';
  LegendLspacing[1] := 4; LegendXshift[1] := 0; LegendYshift[1] := 0;
  LegendLspacing[2] := 0; LegendXshift[2] := 0; LegendYshift[2] := 0;
  LegendLspacing[3] := 0; LegendXshift[3] := 0; LegendYshift[3] := 0;
  LegendLspacing[4] := 0; LegendXshift[4] := 0; LegendYshift[4] := 0;
  LegendLspacing[5] := 0; LegendXshift[5] := 0; LegendYshift[5] := 0;
  {$ifdef Linux}
  MyDocDir := GetEnvironmentVariable('HOME');
  IniFileName := MyDocDir + '/.XYGrapher.ini';
  {$else}
  Reg := TRegistry.Create;
  with Reg do
  try
    RootKey := HKEY_CURRENT_USER;
    OpenKey('Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders', False);
    MyDocDir := ReadString('Personal');// lpszPath; // Modified 02/01/05
    CloseKey;
  finally
    Free;
  end;
  S := ExtractFilePath(MyDocDir) + '\Application Data';
  if DirectoryExists(S) then
    IniFileName := S + '\XYGrapher.ini'
  else
    IniFileName := 'C:\Program Files\XYGrapher\XYGrapher.ini';
  {$endif}
end;

procedure TXYGrapher.SaveFigFile(FileName: String);
var
  i : Integer;
  SL: TStringList;
begin
  SL := TStringList.Create;
  try
  with SL do
  begin
    Clear;
    Add('Information file for XYGrapher ver.1');
    Add('');
    Add(IntToStr(NumDataSeries));
    for i := 1 to NumDataSeries do
    begin
      Add('');
      Add(DataSeries[i]);
      Add(IntToStr(Xb[i]) + ' ' + IntToStr(Xt[i]) + ' ' + IntToStr(Yl[i]) + ' ' + IntToStr(Yr[i]) + ' ' + IntToStr(ReadStart[i]) + ' ' + IntToStr(ReadEnd[i]));
      Add(IntToStr(LineStyle[i]) + '  ' + FloatToStr(LineWidth[i]) + '  ' + IntToStr(LineColor[i]) + ' ' + IntToStr(MarkStyle[i]) + '  ' + FloatToStr(MarkSize[i]) + '  ' + IntToStr(MarkColor[i]));
      Add(FloatToStr(Xmult[i]) + '   ' + FloatToStr(Xoffset[i]) + '   ' + FloatToStr(Ymult[i]) + '   ' + FloatToStr(Yoffset[i]));
    end;
    Add('');
    Add('[Frame]');
    Add(IntToStr(Height) + ' ' + IntToStr(Width) + ' ' + IntToStr(Lmargin) + ' ' + IntToStr(Tmargin) + ' ' + FloatToStr(FLineWidth) + ' ' + IntToStr(NumFontSize));
    Add(NumFontName);
    Add('');
    Add('[X-axis]');
    Add(FloatToStr(Max[1]) + '  ' + FloatToStr(Min[1]) + '  ' + IntToStr(Division[1]) + '  ' + IntToStr(Minordivision[1]) + '  ' + IntToStr(logarithm[1]) + '  ' + FloatToStr(NumeralsXshift[1]) + '  ' + FloatToStr(NumeralsYshift[1]) + '  ' + IntToStr(ZeroLine[1]));
    Add('[U-axis]');
    Add(FloatToStr(Max[2]) + '  ' + FloatToStr(Min[2]) + '  ' + IntToStr(Division[2]) + '  ' + IntToStr(Minordivision[2]) + '  ' + IntToStr(logarithm[2]) + '  ' + FloatToStr(NumeralsXshift[2]) + '  ' + FloatToStr(NumeralsYshift[2]) + '  ' + IntToStr(ZeroLine[2]));
    Add('[Y-axis]');
    Add(FloatToStr(Max[3]) + '  ' + FloatToStr(Min[3]) + '  ' + IntToStr(Division[3]) + '  ' + IntToStr(Minordivision[3]) + '  ' + IntToStr(logarithm[3]) + '  ' + FloatToStr(NumeralsXshift[3]) + '  ' + FloatToStr(NumeralsYshift[3]) + '  ' + IntToStr(ZeroLine[3]));
    Add('[R-axis]');
    Add(FloatToStr(Max[4]) + '  ' + FloatToStr(Min[4]) + '  ' + IntToStr(Division[4]) + '  ' + IntToStr(Minordivision[4]) + '  ' + IntToStr(logarithm[4]) + '  ' + FloatToStr(NumeralsXshift[4]) + '  ' + FloatToStr(NumeralsYshift[4]) + '  ' + IntToStr(ZeroLine[4]));
    Add('');
    Add('[Titles & legends]');
    Add(IntToStr(NumLegend));
    Add('');
    for i := 1 to NumLegend do
    begin
      Add(Legend[i] );
      Add(FloatToStr(LegendLspacing[i]) + '  ' + FormatFloat('##0.0', LegendXshift[i]) + '  ' + FormatFloat('##0.0', LegendYshift[i]));
    end;
    SaveToFile(FileName + '.xyg');
  end;
  finally
    SL.Free;
  end;
end;

procedure TXYGrapher.ExecuteXYGrapher(FileName: String);
var
  S: String;
{$ifdef MSWINDOWS}
  StartInfo: TStartupInfo;
  ProcInfo: TProcessInformation;
  IniFile: TIniFile;
  Path_of_GraphApp: String;
{$endif}
begin
{$ifdef Linux}
 // IniFileName := MyDocDir + '/.XYGrapher.ini';
  S := '/usr/bin/xygrapher';
{$else}
  IniFile := TIniFile.Create(IniFileName);
  try
    Path_of_GraphApp := IniFile.ReadString('Options', 'Path_of_GraphApp', '');
  finally
    IniFile.Free;
  end;
  S := Path_of_GraphApp + 'XYGrapher.exe';
  if not FileExists(S) then
    S := 'C:\Program Files\XYGrapher\XYGrapher.exe';
{$endif}
  if not FileExists(S) then
  begin
    ShowMessage('Please install "XYGrapher" before calling this function.');
    Exit;
  end;
{$ifdef Linux}
  Libc.system(PChar('xygrapher "' + FileName + '.xyg"'));
{$else}
  FillChar(StartInfo, SizeOf(TStartupInfo), 0);
  FillChar(ProcInfo, SizeOf(TProcessInformation), 0);
  StartInfo.dwFlags:= STARTF_USESHOWWINDOW;
  StartInfo.wShowWindow:= SW_SHOWNORMAL;
  CreateProcess(nil,PChar(Path_of_GraphApp + 'XYGrapher.exe ' + FileName + '.xyg'),nil,nil,false,
                NORMAL_PRIORITY_CLASS,nil,nil,StartINfo,ProcINfo);
  GetForegroundWindow;
{$endif}
end;

procedure TXYGrapher.SetFormSize(Width, Height: Integer);
var IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(IniFileName);
  try
    IniFile.WriteInteger('Options', 'Height', Width);
    IniFile.WriteInteger('Options', 'Width' , Height);
  finally
    IniFile.Free;
  end;
end;


end.
 
