unit MyMsgDlg;

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QGraphics, QControls, QForms, 
  QDialogs, QStdCtrls, QExtCtrls;
{$else}  
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;
{$endif}

type
  TMyMessageDlg = class(TForm)
    procedure btnHelpClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private  }
    MsgBrd : TMemo;
    Panel1 : TPanel;
    btnYes, btnNo, btnOK, btnCancel, btnHelp : TButton;
  public
    function MsgBox(Msg, Title, Option: String; HelpContextID: Integer): String;
    function ReceiveString(ACaption, APrompt, TheUnit: String; Default: String; TheWidth, HelpContextID: Integer):String ;
    function ReceiveFloat(ACaption, APrompt, TheUnit: String; Default: Double; TheWidth, HelpContextID: Integer): Double;
    function ReceiveInteger(ACaption, APrompt, TheUnit: String; Default: Integer; TheWidth, HelpContextID: Integer): Integer;
  end;

var
  MyMessageDlg: TMyMessageDlg;

implementation

{$R *.xfm}

procedure TMyMessageDlg.btnHelpClick(Sender: TObject);
begin
  {$ifdef Linux}
  InvokeHelp;
  {$else}
  Keybd_Event(VK_F1, 0, 0, 0);
  {$endif}
end;

function TMyMessageDlg.MsgBox(Msg, Title, Option: String; HelpContextID: Integer): String;
var
  x, f1, f2, i: Integer;
  str: String;
begin
  MsgBrd := TMemo.Create(Self);
  with MsgBrd do
  begin
    Parent := Self;
    Left := 24;
    Top := 16;
    Width := 361;
    BorderStyle := bsNone;
    Color := clBtnFace;
    ReadOnly := True;
    TabOrder := 1;
    Lines.Strings[0] := '';
    Text := Msg;
    f1 := Length(WideString(Msg));
    f2 := Length(Msg);
    if (f1 / f2) > 0.9 then
    begin
      Font.Charset := ANSI_CHARSET;
      Font.Name     := 'Verdana';
    end
    else
    begin
      {$ifdef Linux}
      Font.Charset := fcsSet_Ja;
      Font.Name := 'Gothic [jis]';
      {$else}
      Font.Charset := 128;
      {$endif}
      Font.Size := 10;
    end
  end;
  for i := 1 to MsgBrd.Lines.Count - 1 do
  begin
    str := MsgBrd.Lines[i];
    if (str[1] = ',') or (str[1] = '.') or (str[1] = '"') then
      MsgBrd.Width := MsgBrd.Width + 6;
    str := Copy(str, 1, 2);
    if (str = '') or (str = '') or
       (str = '') or (str = '') or (str = '') then
      MsgBrd.Width := MsgBrd.Width + 12;
  end;
  x := 0;
  MsgBrd.Height := 11 * (MsgBrd.Lines.Count + 1) + 10;  ; //(Font.Size + 1)
  Panel1.Top := MsgBrd.Top + MsgBrd.Height + 10;
  {$ifdef Linux}
  Height := Panel1.Top + Panel1.Height + 5;
  {$else}
  Height := Panel1.Top + Panel1.Height + 20;
  {$endif}
  Width := 440;
  Panel1.Width := Width - 2;
  Caption := Title;
  btnOK.Visible     := False;
  btnHelp.Visible   := False;
  {
  btnYes.Visible    := False;
  btnNo.Visible     := False;
  btnCancel.Visible := False;
  btnOk.Width     := 65;
  btnHelp.Width   := 65;
  }
  if AnsiPos('Yes', Option)    <> 0 then  x := x + 85;
  if AnsiPos('No', Option)     <> 0 then  x := x + 85;
  if AnsiPos('OK', Option)     <> 0 then  x := x + 85;
  if AnsiPos('Cancel', Option) <> 0 then  x := x + 85;
  if AnsiPos('Help', Option)   <> 0 then  x := x + 65;
  x := (Width - x) div 2 - 85;
  if AnsiPos('Yes', Option) <> 0 then
  begin
    x := x + 85;
    btnYes := TButton.Create(Self);
    Application.ProcessMessages;
    with btnYes do
    begin
      Parent := Panel1;
      Left := x;
      Top := 0;
      Width := 65;
      Height := 22;
      Caption := '&Yes';
      ModalResult := 6;
      TabOrder := 0;
    end;
  end;
  if AnsiPos('OK', Option) <> 0 then
  begin
    btnOK.Visible := True;
    btnOK.Width := 65;
    x := x + 85;
    btnOK.Left := x;
  end;
  if AnsiPos('Cancel', Option) <> 0 then
  begin
    x := x + 85;
    btnCancel := TButton.Create(Self);
    Application.ProcessMessages;
    with btnCancel do
    begin
      Parent := Panel1;
      Left := x;
      Top := 0;
      Width := 65;
      Height := 22;
      Cancel := True;
      Caption := '&Cancel';
      ModalResult := 2;
      TabOrder := 3;
    end
  end;
  if AnsiPos('No', Option) <> 0 then
  begin
    x := x + 85;
    btnNo := TButton.Create(Self);
    Application.ProcessMessages;
    with btnNo do
    begin
      Parent := Panel1;
      Left := x;
      Top := 0;
      Width := 65;
      Height := 22;
      Caption := '&No';
      ModalResult := 7;
      TabOrder := 1;
    end;
  end;
  if AnsiPos('Help', Option) <> 0 then
  begin
    x := x + 85;
    btnHelp.Visible := True;
    btnHelp.Width := 65;
    btnHelp.Left := x;
    HelpContext := HelpContextID;
  end;
  Case ShowModal of
    mrYes   : Result := 'Yes';
    mrNo    : Result := 'No';
    mrOK    : Result := 'OK';
    mrCancel: Result := 'Cancel';
  end;
  if AnsiPos('Yes'   , Option) <> 0 then btnYes.Free;
  if AnsiPos('No'    , Option) <> 0 then btnNo.Free;
 // if AnsiPos('OK'    , Option) <> 0 then btnOK.Free;
  if AnsiPos('Cancel', Option) <> 0 then btnCancel.Free;
 // if AnsiPos('Help'  , Option) <> 0 then btnHelp.Free;
  MsgBrd.Free;
end;

function TMyMessageDlg.ReceiveFloat(ACaption, APrompt, TheUnit: String; Default: Double; TheWidth, HelpContextID: Integer): Double;
begin
 Result := StrToFloat(ReceiveString(ACaption, APrompt, TheUnit, FloatToStr(Default), TheWidth, HelpContextID));
end;

function TMyMessageDlg.ReceiveInteger(ACaption, APrompt, TheUnit: String; Default: Integer; TheWidth, HelpContextID: Integer): Integer;
begin
 Result := StrToInt(ReceiveString(ACaption, APrompt, TheUnit, IntToStr(Default), TheWidth, HelpContextID));
end;

function TMyMessageDlg.ReceiveString(ACaption, APrompt, TheUnit: String; Default: String; TheWidth, HelpContextID: Integer):String ;
var
  Label1, Label2 : TLabel;
  f1, f2: Integer;
  TextBox : TEdit;
begin
  HelpContext := HelpContextID;
  Caption := ACaption;
  f1 := Length(WideString(APrompt));
  f2 := Length(APrompt);
  if (f1 / f2) > 0.85 then
  begin
    Font.Charset := ANSI_CHARSET;
    Font.Name     := 'Verdana';
  end
  else
  begin
    {$ifdef Linux}
    Font.Charset := fcsSet_Ja;
    Font.Name := 'Gothic [jis]';
    {$else}
    Font.Charset := 128;
    Font.Name := 'MS På';
    {$endif}
    Font.Size := 9;
    {
    btnCancel.Height := 26;
    btnHelp.Height   := 26;
    btnNo.Height     := 26;
    btnOK.Height     := 26;
    btnYes.Height    := 26;
    }
  end;
  TextBox := TEdit.Create(Self);
  TextBox.Parent := Self;
  TextBox.Top := 18;
  Label1 := TLabel.Create(Self);
  with Label1 do
  begin
    Parent := Self;
    Height := 14;
    Width  := 100;
    Top    := 20;
    Left   := 16;
    Caption := APrompt;
  end;
  Label2 := TLabel.Create(Self);
  Label2.Parent := Self;
  Label2.Height := Label1.Height;
  if TheUnit <> '' then
  begin
    TextBox.Left := Label1.Left + Label1.Width + 5;
    TextBox.Width := TheWidth;
    Label2.Top := Label1.Top;
    Label2.Left := TextBox.Left + TextBox.Width + 5;
    Label2.Caption := TheUnit;
    Width := Label2.Left + Label2.Width + 24;
  end
  else
  begin
    TextBox.Left := 16;
    TextBox.Width := Width;
    TextBox.Top := Label1.Top + Label1.Height + 8;
    Width := TextBox.Left + TextBox.Width + 24;
  end;
  Panel1.Width := Width - 2;
  TextBox.Text := Default;
  Panel1.Top := TextBox.Top + TextBox.Height + 15;
  HelpContext := HelpContextID;
  btnOk.Width     := Width div 5;
  btnHelp.Width   := Width div 5;
  btnOk.Left     := Trunc(Width * 1.25) div 5;
  btnHelp.Left   := Trunc(Width * 2.75) div 5;
  {$ifdef Linux}
  Height := Panel1.Top + Panel1.Height + 5;
  {$else}
  Height := Panel1.Top + Panel1.Height + 22;
  {$endif}
  if ShowModal = mrOK then
    Result := TextBox.Text
  else
    Result := TextBox.Text;
  Label1.Free;
  Label2.Free;
  TextBox.Free;
end;

procedure TMyMessageDlg.FormCreate(Sender: TObject);
begin
  Left := 229;
  Top := 113;
  HelpContext := 1;
  {$ifdef Linux}
  BorderStyle := fbsDialog;
  {$else}
  BorderStyle := bsDialog;
  OldCreateOrder := False;
  {$endif}
  ClientHeight := 104;
  ClientWidth := 413;
  Color := clBtnFace;
  Font.Charset := ANSI_CHARSET;
  Font.Color := clWindowText;
  Font.Height := -11;
  Font.Name := 'Arial';
  Font.Style := [];
  Position := poScreenCenter;
  Scaled := False;
  OnCreate := FormCreate;
  PixelsPerInch := 96;
  //TextHeight := 14;
  Panel1 := TPanel.Create(Self);
  with Panel1 do
  begin
    Parent := Self;
    Left := 0;
    Width := 800;
    Height := 40;
    BevelOuter := bvNone;
    TabOrder := 0;
  end;
  btnOK := TButton.Create(Self);
  with btnOK do
  begin
    Parent := Panel1;
    Top := 0;
    Width := 65;
    Height := 22;
    Default := True;
    Caption := '&OK';
    ModalResult := 1;
    TabOrder := 2;
  end;
  btnHelp := TButton.Create(Self);
  with btnHelp do
  begin
    Parent := Panel1;
    Top := 0;
    Width := 65;
    Height := 22;
    Caption := '&Help';
    TabOrder := 4;
    OnClick := btnHelpClick;
  end;
end;
end.
