{
_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/

        WASH_1D version 1.34

        Copyright(C) 2000-2003 Haruyuki FUJIMAKI and Mitsuhiro INOUE
        fujimaki@sakura.cc.tsukuba.ac.jp
        URL: http://homepage2.nifty.com/Fujimaki/
        Distributed under the GPL

_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
}

unit Output;

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QGraphics, QControls, QForms,
  QDialogs, QStdCtrls, QExtCtrls, Math, ZipMasterForLinux, IniFiles, Libc;
{$else}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Registry, Math, ZipMstr, {$WARN UNIT_PLATFORM OFF}FileCtrl{$WARN UNIT_PLATFORM ON}, IniFiles;
{$endif}

type
  TOutputResults = class(TComponent)
  private

  public
    Timer1: TTimer;
    constructor Create(AOwner: TComponent); override;
    procedure UpdateClock(Sender: TObject);
    procedure ConvertToRate;
    procedure PrepareAnime;
    procedure Play_or_StopClick;
    procedure RewindClick;
    procedure FastForwardClick;
    procedure SliceRings();
    procedure SetProperty_of_TSM_ObservationFigure(FileName: String);
    procedure SetProperty_of_MSO_ObservationFigure(FileName: String);
    procedure SetProperty_of_MSI_ObservationFigure(FileName: String);
    procedure SetProperty_of_UIM_ObservationFigure(FileName: String);
    procedure SetProperty_of_EM_ObservationFigure(FileName: String);
    procedure SetProperty_of_TherConCurve(layer: Integer; FileName: String);
    procedure SetProperty_of_AlbedoCurve(layer: Integer; FileName: String);
    procedure SetProperty_of_EnCurve(layer: Integer; FileName: String);
    procedure SetProperty_of_ProfileFigure(FileName: String);
end;

var
  Msg: array[1..40] of String;

implementation

uses WASH1dMain, Simulation, RC_params, K_Params, FigFileMaker;

constructor TOutputResults.Create(AOwner: TComponent);
var
  IniFile: TIniFile;
  ResourceFileName: String;
  i: Integer;
begin
  inherited Create(AOwner);
  DemonStration := False;
  Timer1 := TTimer.Create(self);
  Timer1.Enabled := False;
  Timer1.OnTimer := UpdateClock;
  Timer1.Interval := 500;
  Msg[1] := '[r][b]Fig.1[ ] Comparison of observed and optimized\n         [r]outflow and pressure head\n';
  Msg[2] := 'Time(h)\n';
  Msg[3] := '';
  Msg[4] := 'Cumulative outflow  [r][i]Q[ ][a](cm)\n';
  Msg[5] := 'Pressure head [s]y[a](cm)\n';
  Msg[6] := '%1 [1]Observed\n%2 [1]Optimized\n%3 [1][s]y[a] at the bottom\n';
  Msg[7] := '[r][b]Fig.1[ ] Comparison of observed and optimized inflow\n';
  Msg[8] := 'Time(h)\n';
  Msg[9] := '';
  Msg[10]:= 'Cumulative inflow  [r][i]Q[ ][a](cm)\n';
  Msg[11]:= 'Pressure head [s]y[a](cm)\n';
  Msg[12]:= '%1 [1]Observed\n%2 [1]Optimized\n%3 [1][s]y[a] at the bottom\n';
  Msg[13]:= '[r][b]Fig.1[ ] Comparison of observed and optimized inflow amount\n';
  Msg[14]:= 'Time(h)\n';
  Msg[15]:= '';
  Msg[16]:= 'Cumulative inflow  [r][i]Q[ ][a](cm)\n';
  Msg[17]:= 'Depth of infiltrating front (cm)\n';
  Msg[18]:= '%1 [1]Observed\n%2 [1]Optimized\n%3 Depth of front\n';
  Msg[19]:= 'Thermal conductivity of ';
  Msg[20]:= '  V.W.C.     Thermal Cond.(W/cm/K) ';
  Msg[21]:= '[r][b]Fig._[ ]  Thermal conductivity of ';
  Msg[22]:= 'Degree of saturation [s]q/q[a][_]s[-][a]\n';
  Msg[23]:= '';
  Msg[24]:= 'Thermal conductivity [r][i]K[_]h[-][ ][a](W/cm/K)\n';
  Msg[25]:= '';
  Msg[26]:= 'Dependence of albedo on water content of ';
  Msg[27]:= '  V.W.C.     Albedo';
  Msg[28]:= '[r][b]Fig._[ ]  Thermal conductivity of ';
  Msg[29]:= 'Volumetric water content [s]q[a] (cm[^]3[-]/cm[^]3[-])\n';
  Msg[30]:= '';
  Msg[31]:= 'Albedo  [s]a\n';
  Msg[32]:= '';
  Msg[33]:= '[r][b]Fig._[ ]  Dependence of enhancement factor for thermal vapor diffusion\n[r]          of ' + SoilName[j] + '\n';
  Msg[34]:= 'Degree of saturation [s]q/q[a][_][i]sat\n';
  Msg[35]:= '';
  Msg[36]:= 'Enhancement factor  [s]h\n';
  Msg[37]:= '';
  ResourceFileName := ChangeFileExt(Application.ExeName, '.lng');
  if FileExists(ResourceFileName) then
  begin
    IniFile := TIniFile.Create(ResourceFileName);
    with IniFile do
      try
        for i := 1 to 37 do
          Msg[i] := StringReplace(ReadString('OutputResults', 'Msg' + IntToStr(i), ''), '<br>', #13#10, [rfReplaceAll, rfIgnoreCase]);
      finally
        Free;
      end;
  end;
end;
//----------------------------------------------------------
procedure TOutputResults.ConvertToRate;
var
  i: Integer;
  f: Textfile;
  SL : TStringList;
  ql_top: array[0..500] of Single;
begin
  AssignFile(f, CaseName + '_flux.out' );
  //FileMode := 0;
  Reset(f);
  for i :=1 to 2 do readln(f,dumstr);
  i := 0;
  repeat
    i := i + 1;
    read(f, time[i], CumE[i], CumD[i], CumI[i], Front[i]);
    Evap[i]   := (CumE[i] - CumE[i-1]) / (time[i] - time[i-1]) * 864000 / t_unit;
    ql_bot[i] := - (CumD[i] - CumD[i-1]) / (time[i] - time[i-1]) * 864000 / t_unit;
    ql_top[i] := - (CumI[i] - CumI[i-1]) / (time[i] - time[i-1]) * 864000 / t_unit;
  until EOF(f);
  CloseFile(f);
  if JustLoading <> 1 then
  begin
    dumint := i;
    SL := TStringList.Create;
    with SL do
      try
        Add('>>>>>>>>>> Time Change of Fluxes <<<<<<<<<<<') ;
        Add('   time           E             D ');
        for i := 1 to dumint-1 do
          Add(Simulator.SetLengthRJ(FormatFloat('######0.000', (time[i] + time[i-1]) / 2), 11) +'   '+
              Simulator.SetLengthRJ(FormatFloat('#0.00', Evap[i]) , 6) + '   ' +
              Simulator.SetLengthRJ(FormatFloat('###0.00', ql_bot[i]), 8) + '   ' +
              Simulator.SetLengthRJ(FormatFloat('####0.00', ql_top[i]), 8));
        SaveToFile(CaseName + '_rates.out' );
      finally
        SL.Free;
      end;
  end;
end;
//====================  Animation Module =====================================================
procedure TOutputResults.PrepareAnime;
var
  f: TextFile;
  i, i2: Integer;
  SL1, SL2: TStringList;
begin
  with ZipMaster do
  begin
    ZipFileName := CaseName + '_history.zip';
    FSpecArgs.Clear;
    FSpecArgs.Add('*.*');
    ExtrBaseDir := BaseDir + Slash + 'History';
    if FileExists(ZipFileName) then
      Extract;
    DeleteFile(ZipFileName);
  end;
  if FileExists(BaseDir + Slash + 'History' + Slash + ExtractFileName(CaseName) + '_001.out') = True then
  begin
    JustLoading := 1;
    if counter = 0 then
    begin
      // loading of data at the 1st observation point //
      AssignFile(f, CaseName + '_observation1.out' );
      Reset(f);
      for i := 1 to 3 do
        readln(f,dumstr);
      i := 0;
      repeat
        i := i + 1;
        read(f, time[i], Wtime[1,i], Ptime[1,i], Ctime[1, i], Ttime[1, i], AvgWaterContentAbove[1, i]);
      until EOF(f);
      CloseFile(f);
      ConvertToRate;
      //---------------------------------------------//
      Simulator.DetermineSpaceIncrements;
    end
    else
      counter := 0;
    if ((Mode = 2) and (IPET_Type <> 5)) or (l_bc_type = 4) then
    begin
      SL1 := TStringList.Create;
      SL2 := TStringList.Create;
      i2 := 0;
      try
        SL1.LoadFromFile(fl_p_bot);
        for i := 1 to SL1.Count do
        begin
          SL2.CommaText := SL1[i-1];
          try
            t_p_bot[i2] := StrToFloat(SL2[0]);
            Pbot[i2]    := StrToFloat(SL2[1]);
            i2 := i2 + 1;
          except
          end;
        end;
      finally
        SL1.Free;
        SL2.Free;
      end;
      NumPbotData := i2 - 1;
    end;
    Main.Play_or_Stop.Enabled := True;
    Main.Slow.Enabled         := True;
    Main.Rewind.Enabled       := True;
    Main.FastForward.Enabled  := True;
    Main.AnimeCounter.Color := clWindow;
    Main.PlayorStop2.Enabled := True;
    Main.Slow2.Enabled         := True;
    Main.Rewind2.Enabled       := True;
    Main.FastForward2.Enabled  := True;
    Main.StatusBar1.SimpleText := '  ' + comment;
  end;
end;
//----------------------------------------------------------
procedure TOutputResults.Play_or_StopClick;
begin
  GoAhead := True; Fast := 0;
  Timer1.Interval := 250;
  if Main.Play_or_Stop.Caption = '>' then
  begin
    playing := 1;
    Main.Play_or_Stop.Caption := '[]';
    Timer1.Enabled := True
  end
  else
  begin
    playing := 0;
    Main.Play_or_Stop.Caption := '>';
    Timer1.Enabled := False;
  end;
end;
//----------------------------------------------------------
procedure TOutputResults.RewindClick;
begin
  Timer1.Enabled := True;
  if playing = 1 then
    begin
    if GoAhead = False then Timer1.Interval := Timer1.Interval div 2;
    end
  else
    begin
    Play_or_StopClick;
    Fast := 1;
    Timer1.Interval := 100;
    end;
  GoAhead := False;
end;
//----------------------------------------------------------
procedure TOutputResults.FastForwardClick;
begin
  Timer1.Enabled := True;
  if playing = 1 then
    begin
    if GoAhead = True then Timer1.Interval := Timer1.Interval div 2;
    end
  else
    begin
    Play_or_StopClick;
    Fast := 1;
    Timer1.Interval := 100;
    end;
  GoAhead := True;
end;
//----------------------------------------------------------
procedure TOutputResults.UpdateClock(Sender: TObject);
var
i, j: Integer;
f: TextFile;
begin
  Application.ProcessMessages;
  if Fast = 1 then dumint := 4 else dumint := 1;
  if GoAhead = True then
    begin
    if (counter + 1 > t_end / int_rec) and (t_end > 0) then
      if DemonStration = True then
        counter := 0
      else
        begin
        Main.Play_or_StopClick(Sender);
        Exit;
        end;
    counter := counter + dumint;
    end
  else
    begin
    if counter = 0 then
      begin
      Main.Play_or_StopClick(Sender);
      end;
    counter := counter - dumint;
    if counter <= 0 then counter := 0;
    end;
  time_ := counter * int_rec;
  Main.AnimeCounter.Text := FloatToStr(time_);
  if (Fast = 1) and (counter <> 0) then Exit;
  if (playing = 1) or (counter = 0) then
    begin
    for i := 1 to n do dumarray[i] := (z[i] + z[i-1]) / 2;
    dumstr :=  BaseDir + Slash + 'History' + Slash + ExtractFileName(CaseName) + '_' + FormatFloat('000', counter) + '.out';
    if FileExists(dumstr) = False then
      begin
      Main.Play_or_StopClick(Sender);
      t_end := counter * int_rec ;
      Exit;
      end;
    {$I-}
    AssignFile(f, dumstr);
    Reset(f);
    Read(f, dum, avdt, navdt, WCumRError, SCumRError, ECumRError, Hpond);
    for i := 1 to 2 do Readln(f, dumstr);
    for i := 1 to n do
      begin
      Read(f, dum, W[i], P[i], T[i], C[i], dum, ql[i], qv[i]) ;
      T[i] := T[i] + 273.2;
      ql[i] := - ql[i];
      qv[i] := - qv[i];
      end;
    CloseFile(f);
    {$I+}
    Main.Profile;
    Main.ProfileDraw(W,dumarray,n,0,0.6,clBlue);
    Main.ProfileDraw(C,dumarray,n,0,6,$009090);
    //----------------------------------------------------------
    if Heat = 1 then
      Main.ProfileDraw(T, dumarray, n , 273.2, 333.2, $000080FF)
    else
      begin
      if P[n] < -150 then dum := -900 else dum := -150;
      Main.ProfileDraw(P,dumarray, n, dum, 0,clGray);
      end;
    Main.FluxProfile;
    Main.FluxProfileDraw(ql,dumarray, clNavy);
    Main.FluxProfileDraw(qv,dumarray, clAqua);
    j := Mode; Mode := 1;
    Main.History;
    for i := 1 to counter do dumarray[i] := Wtime[1, i];
    Main.HistoryDraw(time, dumarray, 1, counter, -0.6, 0.4, clBlue, psSolid);
    if solute = 1 then
      begin
      for i := 1 to counter do dumarray[i] := Ctime[1, i];
      Main.HistoryDraw(time, dumarray, 1, counter, -12 ,   8, $009090, psSolid);
      end;
    if heat = 1 then
      begin
      for i := 1 to counter do dumarray[i] := Ttime[1, i];
      Main.HistoryDraw(time, dumarray, 1, counter, -60 ,  40, $000080FF, psSolid);
      end;
    for i := 1 to counter do dumarray[i] := Ptime[1, i];
    Main.HistoryDraw(time, dumarray, 1, counter, -600, 400, clTeal, psSolid);
    Main.HistoryDraw(time, ql_bot, 1, counter,  -60,  40, clNavy, psSolid);
    Main.HistoryDraw(time, Evap  , 1, counter,  -60,  40, clAqua, psSolid);
    if l_bc_type = 4 then Main.HistoryDraw(t_P_bot, Pbot, 1, NumPbotData, -600, 400, clTeal, psDot);
    Main.Report;
    Mode := j;
    end;
end;

//===============================================================================================================================
procedure TOutputResults.SetProperty_of_TSM_ObservationFigure(FileName: String);
var
  TSM_ObservationFigure: TXYGrapher;
begin
  TSM_ObservationFigure := TXYGrapher.Create;
  with TSM_ObservationFigure do
  begin
    SetFormSize(600, 750);
    NumDataSeries := 5;
    //----------
    DataSeries[1] := fl_outflow;
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 0; ReadEnd[1] := 0;
    LineStyle[1] := 0; LineWidth[1] := 0.25; LineColor[1] := 0; MarkStyle[1] := 1; MarkSize[1] := 1.5; MarkColor[1] := 16776960;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    //----------
    DataSeries[2] := fl_CumEvap;
    Xb[2] := 1; Xt[2] := 0; Yl[2] := 2; Yr[2] := 0; ReadStart[2] := 0; ReadEnd[2] := 0;
    LineStyle[2] := 0; LineWidth[2] := 0.25; LineColor[2] := 0; MarkStyle[2] := 5; MarkSize[2] := 1.5; MarkColor[2] := 16711680;
    Xmult[2] := 1; Xoffset[2] := 0; Ymult[2] := 1; Yoffset[2] := 0;
    //----------
    DataSeries[3] := fl_P_bot;
    Xb[3] := 1; Xt[3] := 0; Yl[3] := 0; Yr[3] := 2; ReadStart[3] := 0; ReadEnd[3] := 0;
    LineStyle[3] := 2; LineWidth[3] := 0.2; LineColor[3] := 16744576; MarkStyle[3] := 0; MarkSize[3] := 0; MarkColor[3] := 0;
    Xmult[3] := 1; Xoffset[3] := 0; Ymult[3] := 1; Yoffset[3] := 0;
    //----------
    DataSeries[4] := CaseName + '_flux.out';
    Xb[4] := 1; Xt[4] := 0; Yl[4] := 3; Yr[4] := 0; ReadStart[4] := 0; ReadEnd[4] := 0;
    LineStyle[4] := 1; LineWidth[4] := 0.25; LineColor[4] := 8421440; MarkStyle[4] := 0; MarkSize[4] := 0; MarkColor[4] := 0;
    Xmult[4] := 1; Xoffset[4] := 0; Ymult[4] := 1; Yoffset[4] := 0;
    //----------
    DataSeries[5] := CaseName + '_flux.out';
    Xb[5] := 1; Xt[5] := 0; Yl[5] := 2; Yr[5] := 0; ReadStart[5] := 0; ReadEnd[5] := 0;
    LineStyle[5] := 1; LineWidth[5] := 0.25; LineColor[5] := 8421440; MarkStyle[5] := 0; MarkSize[5] := 0; MarkColor[5] := 0;
    Xmult[5] := 1; Xoffset[5] := 0; Ymult[5] := 1; Yoffset[5] := 0;
    //----------
    Height := 60; Width := 120; Lmargin := 30; Tmargin := 20; FLineWidth := 0.3; NumFontSize := 10;
    NumFontName := 'Arial';
    Max[1] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[1] := 0 ; Division[1] := 6; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[2] := 0 ; Division[2] := 6; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 1.5; Min[3] := 0 ; Division[3] := 6; Minordivision[3] := 2; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 0; Min[4] := -200; Division[4] := 4; Minordivision[4] := 2; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 7;
    Legend[1] := '[r][b][4]Fig.1[ ] Comparison of observed and optimized amounts of outflow and evaporation for ' + SoilName[1] +'. \n';
    Legend[2] := 'Time(h)\n';
    Legend[3] := '';
    Legend[4] := 'Cumulative outflow or evaporation (cm)\n';
    Legend[5] := 'Pressure head [s]y[a](cm)\n';
    Legend[6] := '[b]Measured[ ]\n%1 Outflow \n%2 Evaporation\n%3 [i][r]h[a][ ] at the bottom\n';
    Legend[7] := '%4 Optimized\n';
    LegendLspacing[1] :=5.0; LegendXshift[1] :=2.4; LegendYshift[1] :=-1.4;
    LegendLspacing[2] :=  0; LegendXshift[2] :=1.1; LegendYshift[2] := 0.3;
    LegendLspacing[3] :=  0; LegendXshift[3] :=  0; LegendYshift[3] := 0;
    LegendLspacing[4] :=  0; LegendXshift[4] :=  0; LegendYshift[4] := 0;
    LegendLspacing[5] :=  0; LegendXshift[5] :=  0; LegendYshift[5] := 0;
    LegendLspacing[6] :=  5; LegendXshift[6] :=-5.9; LegendYshift[6] :=-26.9;
    LegendLspacing[7] :=  5; LegendXshift[7] :=28.1; LegendYshift[7] :=-26.9;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  TSM_ObservationFigure.Free;
end;
//----------------------------------------------------------
procedure TOutputResults.SetProperty_of_EM_ObservationFigure(FileName: String);
var
  EM_ObservationFigure: TXYGrapher;
begin
  EM_ObservationFigure := TXYGrapher.Create;
  with EM_ObservationFigure do
  begin
    SetFormSize(600, 750);
    NumDataSeries := 5;
    //----------
    DataSeries[1] := fl_pressure;
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 0; ReadEnd[1] := 0;
    LineStyle[1] := 0; LineWidth[1] := 0.25; LineColor[1] := 0; MarkStyle[1] := 11; MarkSize[1] := 1; MarkColor[1] := 32896;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    //----------
    DataSeries[2] := fl_pressure;
    Xb[2] := 1; Xt[2] := 0; Yl[2] := 3; Yr[2] := 0; ReadStart[2] := 0; ReadEnd[2] := 0;
    LineStyle[2] := 0; LineWidth[2] := 0.25; LineColor[2] := 0; MarkStyle[2] := 13; MarkSize[2] := 1; MarkColor[2] := 32896;
    Xmult[2] := 1; Xoffset[2] := 0; Ymult[2] := 1; Yoffset[2] := 0;
    //----------
    DataSeries[3] := CaseName + '_observation1.out';
    Xb[3] := 1; Xt[3] := 0; Yl[3] := 3; Yr[3] := 0; ReadStart[3] := 0; ReadEnd[3] := 0;
    LineStyle[3] := 1; LineWidth[3] := 0.2; LineColor[3] := 8421440; MarkStyle[3] := 0; MarkSize[3] := 0; MarkColor[3] := 0;
    Xmult[3] := 1; Xoffset[3] := 0; Ymult[3] := 1; Yoffset[3] := 0;
    //----------
    DataSeries[4] := CaseName + '_observation2.out';
    Xb[4] := 1; Xt[4] := 0; Yl[4] := 3; Yr[4] := 0; ReadStart[4] := 0; ReadEnd[4] := 0;
    LineStyle[4] := 2; LineWidth[4] := 0.2; LineColor[4] := 8421440; MarkStyle[4] := 0; MarkSize[4] := 0; MarkColor[4] := 0;
    Xmult[4] := 1; Xoffset[4] := 0; Ymult[4] := 1; Yoffset[4] := 0;
    //----------
    DataSeries[5] := fl_CumEvap;
    Xb[5] := 1; Xt[5] := 0; Yl[5] := 0; Yr[5] := 2; ReadStart[5] := 0; ReadEnd[5] := 0;
    LineStyle[5] := 1; LineWidth[5] := 0.2; LineColor[5] := 16776960; MarkStyle[5] := 0; MarkSize[5] := 0; MarkColor[5] := 0;
    Xmult[5] := 1; Xoffset[5] := 0; Ymult[5] := 1; Yoffset[5] := 0;
    //----------
    Height := 70; Width := 100; Lmargin := 30; Tmargin := 20; FLineWidth := 0.3; NumFontSize := 10;
    NumFontName := 'Arial';
    Max[1] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[1] := 0 ; Division[1] := 6; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[2] := 0 ; Division[2] := 6; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 0; Min[3] := -800 ; Division[3] := 4; Minordivision[3] := 2; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 2; Min[4] := 0    ; Division[4] := 4; Minordivision[4] := 2; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 7;
    Legend[1] := '[r] Fig. Comparison of measured and optimized pressure head during evaporation experiment.\n';
    Legend[2] := 'Time(h)\n';
    Legend[3] := '';
    Legend[4] := 'Pressure head [s]y[a](cm)\n';
    Legend[5] := 'Cumulative evaporation  [s]S[r]E[a](cm)\n';
    Legend[6] := 'Measured\n%1 [r][i]z[/i] = ' + FloatToStr(z_obs[1]) + 'cm\n%2 [i]z[/i] = ' + FloatToStr(z_obs[2]) + 'cm\nOptimized\n%3 [r][i]z[/i] = ' + FloatToStr(z_obs[1]) + 'cm\n%4 [i]z[/i] = ' + FloatToStr(z_obs[2]) + 'cm\n';
    Legend[7] := '%5 Evaporation\n';
    LegendLspacing[1] :=5.0; LegendXshift[1] :=2.4; LegendYshift[1] :=-1.4;
    LegendLspacing[2] :=  0; LegendXshift[2] :=1.1; LegendYshift[2] := 0.3;
    LegendLspacing[3] :=  0; LegendXshift[3] :=  0; LegendYshift[3] := 0;
    LegendLspacing[4] :=  0; LegendXshift[4] :=  0; LegendYshift[4] := 0;
    LegendLspacing[5] :=  0; LegendXshift[5] :=  0; LegendYshift[5] := 0;
    LegendLspacing[6] :=  5; LegendXshift[6] :=-45; LegendYshift[6] :=-22;
    LegendLspacing[7] :=  5; LegendXshift[7] :=-19; LegendYshift[7] :=22;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  EM_ObservationFigure.Free;
end;
//----------------------------------------------------------
procedure TOutputResults.SetProperty_of_MSO_ObservationFigure;
var
  MSO_ObservationFigure : TXYGrapher;
  i: Integer;
begin
  MSO_ObservationFigure := TXYGrapher.Create;
  with MSO_ObservationFigure do
  begin
    SetFormSize(600, 750);
    if tensio <> 0 then NumDataSeries := 5 else NumDataSeries := 4;
    //----------
    DataSeries[1] := fl_outflow;
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 0; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 8421440; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    //----------
    DataSeries[2] := CaseName + '_flux.out';
    Xb[2] := 1; Xt[2] := 0; Yl[2] := 3; Yr[2] := 0; ReadStart[2] := 0; ReadEnd[2] := 0;
    LineStyle[2] := 1; LineWidth[2] := 0.25; LineColor[2] := 16711680; MarkStyle[2] := 0; MarkSize[2] := 0;
    Xmult[2] := 1; Xoffset[2] := 0; Ymult[2] := 1; Yoffset[2] := 0;
    //----------
    DataSeries[3] := fl_P_bot;
    Xb[3] := 1; Xt[3] := 0; Yl[3] := 0; Yr[3] := 2; ReadStart[3] := 0; ReadEnd[3] := 0;
    LineStyle[3] := 2; LineWidth[3] := 0.2; LineColor[3] := 12615680; MarkStyle[3] := 0; MarkSize[3] := 0;
    Xmult[3] := 1; Xoffset[3] := 0; Ymult[3] := 1; Yoffset[3] := 0;
    //----------
    DataSeries[4] := CaseName + '_observation1.out';
    Xb[4] := 1; Xt[4] := 0; Yl[4] := 0; Yr[4] := 3; ReadStart[4] := 0; ReadEnd[4] := 0;
    LineStyle[4] := 1; LineWidth[4] := 0.25; LineColor[4] := 16711680; MarkStyle[4] := 0; MarkSize[4] := 0;
    Xmult[4] := 1; Xoffset[4] := 0; Ymult[4] := 1; Yoffset[4] := 0;
    //----------
    DataSeries[5] := fl_pressure;
    Xb[5] := 1; Xt[5] := 0; Yl[5] := 0; Yr[5] := 2; ReadStart[5] := 0; ReadEnd[5] := 0;
    LineStyle[5] := 1; LineWidth[5] := 0.25; LineColor[5] := 8421440; MarkStyle[5] := 0; MarkSize[5] := 0;
    Xmult[5] := 1; Xoffset[5] := 0; Ymult[5] := 1; Yoffset[5] := 0;
    //----------
    Height := 60; Width := 120; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    Max[1] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[1] := 0 ; Division[1] := 6; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[2] := 0 ; Division[2] := 6; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 1.5; Min[3] := 0 ; Division[3] := 5; Minordivision[3] := 2; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 0; Min[4] := -800; Division[4] := 4; Minordivision[4] := 2; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 6;
    for i := 1 to 6 do
      Legend[i] := Msg[i];
    LegendLspacing[1] :=4.5; LegendXshift[1] :=  0; LegendYshift[1] := 0;
    LegendLspacing[2] :=  0; LegendXshift[2] :=1.1; LegendYshift[2] := 0.3;
    LegendLspacing[3] :=  0; LegendXshift[3] :=  0; LegendYshift[3] := 0;
    LegendLspacing[4] :=  0; LegendXshift[4] :=  0; LegendYshift[4] := 0;
    LegendLspacing[5] :=  0; LegendXshift[5] :=  0; LegendYshift[5] := 0;
    LegendLspacing[6] :=  5; LegendXshift[6] :=-20; LegendYshift[6] := 10;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  MSO_ObservationFigure.Free;
end;
//----------------------------------------------------------
procedure TOutputResults.SetProperty_of_MSI_ObservationFigure;
var
  MSI_ObservationFigure : TXYGrapher;
  i: Integer;
begin
  MSI_ObservationFigure := TXYGrapher.Create;
  with MSI_ObservationFigure do
  begin
    SetFormSize(600, 750);
    NumDataSeries := 3;
    //----------
    DataSeries[1] := fl_outflow;
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 0; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 8421440; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    //----------
    DataSeries[2] := CaseName + '_flux.out';
    Xb[2] := 1; Xt[2] := 0; Yl[2] := 3; Yr[2] := 0; ReadStart[2] := 0; ReadEnd[2] := 0;
    LineStyle[2] := 1; LineWidth[2] := 0.25; LineColor[2] := 16711680; MarkStyle[2] := 0; MarkSize[2] := 0;
    Xmult[2] := 1; Xoffset[2] := 0; Ymult[2] := -1; Yoffset[2] := 0;
    //----------
    DataSeries[3] := fl_P_bot;
    Xb[3] := 1; Xt[3] := 0; Yl[3] := 0; Yr[3] := 2; ReadStart[3] := 0; ReadEnd[3] := 0;
    LineStyle[3] := 2; LineWidth[3] := 0.2; LineColor[3] := 12615680; MarkStyle[3] := 0; MarkSize[3] := 0;
    Xmult[3] := 1; Xoffset[3] := 0; Ymult[3] := 1; Yoffset[3] := 0;
    //----------
    Height := 60; Width := 120; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    Max[1] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[1] := 0 ; Division[1] := 6; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[2] := 0 ; Division[2] := 6; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 1.25; Min[3] :=    0; Division[3] := 5; Minordivision[3] := 2; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] :=    0; Min[4] := -250; Division[4] := 5; Minordivision[4] := 2; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 6;
    for i := 1 to 6 do
      Legend[i] := Msg[i+6];
    LegendLspacing[1] :=4.5; LegendXshift[1] :=  0; LegendYshift[1] := 0;
    LegendLspacing[2] :=  0; LegendXshift[2] :=1.1; LegendYshift[2] := 0.3;
    LegendLspacing[3] :=  0; LegendXshift[3] :=  0; LegendYshift[3] := 0;
    LegendLspacing[4] :=  0; LegendXshift[4] :=  0; LegendYshift[4] := 0;
    LegendLspacing[5] :=  0; LegendXshift[5] :=  0; LegendYshift[5] := 0;
    LegendLspacing[6] :=  5; LegendXshift[6] :=-20; LegendYshift[6] := 10;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  MSI_ObservationFigure.Free;
end;
//----------------------------------------------------------------------
procedure TOutputResults.SetProperty_of_UIM_ObservationFigure;
var
  UIM_ObservationFigure : TXYGrapher;
  i: Integer;
begin
  UIM_ObservationFigure := TXYGrapher.Create;
  with UIM_ObservationFigure do
  begin
    SetFormSize(600, 750);
    if fl_front <> '' then
      NumDataSeries := 4
    else
      NumDataSeries := 3;
    //----------
    DataSeries[1] := fl_outflow;
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 0; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 8421440; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    //----------
    DataSeries[2] := CaseName + '_flux.out';
    Xb[2] := 1; Xt[2] := 0; Yl[2] := 3; Yr[2] := 0; ReadStart[2] := 0; ReadEnd[2] := 0;
    LineStyle[2] := 2; LineWidth[2] := 0.25; LineColor[2] := 8421440; MarkStyle[2] := 0; MarkSize[2] := 0;
    Xmult[2] := 1; Xoffset[2] := 0; Ymult[2] := -1; Yoffset[2] := 0;
    //----------
    DataSeries[3] := CaseName + '_flux.out';
    Xb[3] := 1; Xt[3] := 0; Yl[3] := 0; Yr[3] := 5; ReadStart[3] := 0; ReadEnd[3] := 0;
    LineStyle[3] := 2; LineWidth[3] := 0.2; LineColor[3] := 12615680; MarkStyle[3] := 0; MarkSize[3] := 0;
    Xmult[3] := 1; Xoffset[3] := 0; Ymult[3] := 1; Yoffset[3] := 0;
    //----------
    if fl_front <> '' then
    begin
      DataSeries[4] := fl_front;
      Xb[4] := 1; Xt[4] := 0; Yl[4] := 0; Yr[4] := 2; ReadStart[4] := 0; ReadEnd[4] := 0;
      LineStyle[4] := 1; LineWidth[4] := 0.2; LineColor[4] := 12615680; MarkStyle[4] := 0; MarkSize[4] := 0;
      Xmult[4] := 1; Xoffset[4] := 0; Ymult[4] := 1; Yoffset[4] := 0;
    end;
    //----------
    Height := 60; Width := 120; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    Max[1] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[1] := 0 ; Division[1] := 6; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 6 * Ceil(t_end * t_unit / (3600 * 6)) ; Min[2] := 0 ; Division[2] := 6; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 24; Min[3] :=   0; Division[3] := 6; Minordivision[3] := 2; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] :=  0; Min[4] :=  30; Division[4] := 6; Minordivision[4] := 2; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 6;
    for i := 1 to 6 do
      Legend[i] := Msg[i+12];
    LegendLspacing[1] :=4.5; LegendXshift[1] :=  0; LegendYshift[1] := 0;
    LegendLspacing[2] :=  0; LegendXshift[2] :=1.1; LegendYshift[2] := 0.3;
    LegendLspacing[3] :=  0; LegendXshift[3] :=  0; LegendYshift[3] := 0;
    LegendLspacing[4] :=  0; LegendXshift[4] :=  0; LegendYshift[4] := 0;
    LegendLspacing[5] :=  0; LegendXshift[5] :=  0; LegendYshift[5] := 0;
    LegendLspacing[6] :=  5; LegendXshift[6] :=-20; LegendYshift[6] := 10;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  UIM_ObservationFigure.Free;
end;
//----------------------------------------------------------
procedure TOutputResults.SetProperty_of_ProfileFigure(FileName: String);
var
  f : TextFile;
  i: Integer;
  ProfileFigure : TXYGrapher;
begin
  ProfileFigure := TXYGrapher.Create;
  with ProfileFigure do
  begin
    SetFormSize(500, 500);
    NumDataSeries := 1;
    //----------
    DataSeries[1] := CaseName + '_profile.out';
    Xt[1] := 0; Yl[1] := 1; Yr[1] := 0; ReadStart[1] := 7; ReadEnd[1] := 0;
    if (Mode = 1) or (Solute = 0) then
    begin
      Xb[1] := 2;
      LineColor[1] := 16776960;
    end
    else
    begin
      Xb[1] := 4;
      LineColor[1] := 43775;
    end;
    LineStyle[1] := 1; LineWidth[1] := 0.2; MarkStyle[1] := 0; MarkSize[1] := 1.5; MarkColor[1] := LineColor[1];
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    //----------
    if (IPET_type = 1) or (IPET_type = 5) or (IPET_type = 11) then
    begin
      NumDataSeries := 2;
      dumstr := CaseName + '_MeasuredProfile.csv';
      DataSeries[2] := dumstr;
      Xb[2] := 2; Xt[2] := 0; Yl[2] := 1; Yr[2] := 0; ReadStart[2] := 0; ReadEnd[2] := 0;
      LineStyle[2] := 0; LineWidth[2] := 0.25; LineColor[2] := 0; MarkStyle[2] := 11; MarkSize[2] := 1.5;
      Xmult[2] := 1; Xoffset[2] := 0; Ymult[2] := 1; Yoffset[2] := 0;
      //---------
      AssignFile(f, dumstr);
      Rewrite(f);
      if IPET_type <> 11 then
      begin
        Writeln(f, 'Measured water content profile');
        Writeln(f, ' z(cm)   VWC');
        MarkColor[2] := 16776960;
      end
      else
      begin
        Writeln(f, 'Measured relative concentration profile');
        Writeln(f, ' z(cm)   C/Ci');
        MarkColor[2] := 43775;
      end;
      for i := 1 to NumSampling do
        if IPET_type <> 11 then
          Writeln(f, FormatFloat('#0.##', zm[i]) + ', ' + FormatFloat('0.####', Wm[i]))
        else
          Writeln(f, FormatFloat('#0.##', zms[i]) + ', ' + FormatFloat('0.####', Cm[i]));
      CloseFile(f);
    end;
    //----------
    Height := 65; Width := 65; Lmargin := 30; Tmargin := 20; FLineWidth := 0.3; NumFontSize := 10;
    NumFontName := 'Arial';
    if IPET_type <> 11 then
      dumint := Trunc(W[n] * 10) + 1
    else
      dumint := 10;
    Max[1] := 0.1 * dumint; Min[1] := 0; Division[1] := dumint; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 0.1 * dumint; Min[2] := 0; Division[2] := dumint; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    if z_lb < 25 then
      dumint := 5
    else
      dumint := 25;
    dumint := Ceil(z_lb / dumint) * dumint;
    if IPET_type = 1 then
      dumint := 5;
    Max[3] := 0; Min[3] := dumint; Division[3] := 5; Minordivision[3] := 0; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 0; Min[4] := dumint; Division[4] := 5; Minordivision[4] := 0; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    if IPET_type = 1 then
      NumLegend := 6
    else
      NumLegend := 5;
    case t_unit of
        1: dumstr := 's';
       60: dumstr := 'min';
     3600: dumstr := 'h';
    86400: dumstr := 'd';
    end;
    if IPET_type <> 11 then
    begin
      Legend[1] := '[r][b][4]Fig.1[ ] Water content profile at [i]t[ ] = ' + FloatToStr(t_end) + dumstr + '. \n';
      Legend[2] := 'Volumetric water content [s]q[a] (cm[^]3[-]/cm[^]3[-])\n';
    end
    else
    begin
      Legend[1] := '[r][b][4]Fig.1[ ] Concentration profile at [i]t[ ] = ' + FloatToStr(t_end) + dumstr + '. \n';
      Legend[2] := 'Relative concentration [r][i]C / C[_]i[-][/i][a]\n';
    end;
    Legend[3] := '';
    Legend[4] := 'Depth  [i]z[ ](cm)\n';
    Legend[5] := '';
    Legend[6] := '%2 Measured\n%1 Optimized\n';
    LegendLspacing[1] :=5.0; LegendXshift[1] :=2.4; LegendYshift[1] :=-1.4;
    LegendLspacing[2] :=  0; LegendXshift[2] :=1.1; LegendYshift[2] := 0.3;
    LegendLspacing[3] :=  0; LegendXshift[3] :=  0; LegendYshift[3] := 0;
    LegendLspacing[4] :=  0; LegendXshift[4] := 5.0; LegendYshift[4] := 0;
    LegendLspacing[5] :=  0; LegendXshift[5] :=  0; LegendYshift[5] := 0;
    LegendLspacing[6] :=  5; LegendXshift[6] :=-22; LegendYshift[6] :=17.4;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  ProfileFigure.Free;
end;
//----------------------------------------------------------
procedure TOutputResults.SetProperty_of_TherConCurve(layer: Integer; FileName: String);
var
  i : Integer;
  f : TextFile;
  GX, GY: array[0..50] of Double;
  TherConCurve : TXYGrapher;
begin
  //--- Saving of Curve ---//
  j := layer;
  with Soil do
    for i := 0 to 50 do
    begin
      GX[i] := 0.02 * i;
      GY[i] := Akh[j]  + Bkh[j] * GX[i] - (Akh[j] - Dkh[j]) * exp(-Ckh[j] * Power(GX[i], Ekh[j]));
    end;
  dumstr := ExeDir + Slash + 'SoilProperties' + Slash + SoilName[layer];
  if DirectoryExists(dumstr) = False then CreateDir(dumstr);
  AssignFile(f, dumstr + Slash + SoilName[layer] + '_thc.txt');
  Rewrite(f);
  if LANG = 128 then
    Writeln(f, SoilName[j] + Msg[19])
  else
    Writeln(f, Msg[19] + SoilName[j]);
  Writeln(f, '  Akh       Bkh      Ckh      Dkh      Ekh');
  with Soil do
    Writeln(f, ' ' + FloatToStr(Akh[j]) + '    ' + FloatToStr(Bkh[j]) + '    ' + FloatToStr(Ckh[j]) + '    ' + FloatToStr(Dkh[j]) + '    ' + FloatToStr(Ekh[j]));
  Writeln(f, Msg[20]);
  for i := 0 to 50 do
    Writeln(f, '  ' + FormatFloat('0.0000', GX[i]) + '      ' + FormatFloat('0.00000', GY[i]));
  CloseFile(f);
  //---------------------------------
  TherConCurve := TXYGrapher.Create;
  with TherConCurve do
  begin
    SetFormSize(600, 750);
    NumDataSeries := 1;
    //----------
    DataSeries[1] := dumstr + Slash + SoilName[layer] + '_thc.txt';
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 6; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 33023; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    //----------
    Height := 65; Width := 65; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    Max[1] := 1 ; Min[1] := 0 ; Division[1] := 5; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 1 ; Min[2] := 0 ; Division[2] := 5; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 0.01; Min[3] := 0; Division[3] := 5; Minordivision[3] := 2; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 0.01; Min[4] := 0; Division[4] := 5; Minordivision[4] := 2; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 5;
    for i := 2 to 5 do
      Legend[i] := Msg[i+20];
    if LANG = 128 then
      Legend[1] := SoilName[j] + Msg[21]
    else
      Legend[1] := Msg[21] + SoilName[j] + '\n';
    LegendLspacing[1] := 4; LegendXshift[1] := 0; LegendYshift[1] := 0;
    LegendLspacing[2] := 0; LegendXshift[2] := 0; LegendYshift[2] := 0;
    LegendLspacing[3] := 0; LegendXshift[3] := 0; LegendYshift[3] := 0;
    LegendLspacing[4] := 0; LegendXshift[4] := 0; LegendYshift[4] := -2;
    LegendLspacing[5] := 0; LegendXshift[5] := 0; LegendYshift[5] := 0;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  TherConCurve.Free;
end;
//----------------------------------------------------------
procedure TOutputResults.SetProperty_of_AlbedoCurve(layer: Integer; FileName: String);
var
  i : Integer;
  f : TextFile;
  GX, GY: array[0..50] of Double;
  AlbedoCurve : TXYGrapher;
begin
  j := layer;
  with Soil do
    for i := 0 to 50 do
    begin
      GX[i] := 0.02 * i * Wsat[j];
      GY[i] := AlbedoMin[1] + (AlbedoMax[1] - AlbedoMin[1]) / (1 + Power(Aal[1]* GX[i], Bal[1]));
    end;
  dumstr := ExeDir + Slash + 'SoilProperties' + Slash + SoilName[layer];
  if DirectoryExists(dumstr) = False then CreateDir(dumstr);
  AssignFile(f, dumstr + Slash + SoilName[layer] + '_albedo.txt');
  Rewrite(f);
  if LANG = 128 then
    Writeln(f, SoilName[j] + Msg[26])
  else
    Writeln(f, Msg[26] + SoilName[j]);
  Writeln(f, '  Max   Min    a_al   b_al  Walb');
  with Soil do
    Writeln(f, ' ' + FloatToStr(AlbedoMax[j]) + '   ' + FloatToStr(AlbedoMin[j]) + '   ' + FloatToStr(Aal[j]) + '   ' + FloatToStr(Bal[j]) + '   ' + FloatToStr(ThickAlb[j]));
  Writeln(f, Msg[27]);
  for i := 0 to 50 do
    Writeln(f, '  ' + FormatFloat('0.0000', GX[i]) + '      ' + FormatFloat('0.00000', GY[i]));
  CloseFile(f);
  //---------------------------------
  AlbedoCurve := TXYGrapher.Create;
  with AlbedoCurve do
  begin
    SetFormSize(550, 550);
    NumDataSeries := 1;
    //----------
    DataSeries[1] := dumstr + Slash + SoilName[layer] + '_albedo.txt';
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 6; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 32896; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    //----------
    Height := 65; Width := 65; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    dumint := Trunc(Soil.Wsat[j] * 10) + 1;
    Max[1] := 0.1 * dumint; Min[1] := 0 ; Division[1] := dumint; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 0.1 * dumint; Min[2] := 0 ; Division[2] := dumint; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 0.5; Min[3] := 0; Division[3] := 5; Minordivision[3] := 0; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 0.5; Min[4] := 0; Division[4] := 5; Minordivision[4] := 0; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 5;
    for i := 2 to 5 do
      Legend[i] := Msg[i+27];
    if LANG = 128 then
      Legend[1] := SoilName[j] + Msg[28]
    else
      Legend[1] := Msg[28] + SoilName[j] + '\n';
    LegendLspacing[1] := 4; LegendXshift[1] := 0; LegendYshift[1] := 0;
    LegendLspacing[2] := 0; LegendXshift[2] := 0; LegendYshift[2] := 0;
    LegendLspacing[3] := 0; LegendXshift[3] := 0; LegendYshift[3] := 0;
    LegendLspacing[4] := 0; LegendXshift[4] := 0; LegendYshift[4] := 0;
    LegendLspacing[5] := 0; LegendXshift[5] := 0; LegendYshift[5] := 0;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  AlbedoCurve.Free;
end;
//----------------------------------------------------------
procedure TOutputResults.SetProperty_of_EnCurve(layer: Integer; FileName: String);
var
  i : Integer;
  f : TextFile;
  GX, GY: array[0..50] of Double;
  EnCurve : TXYGrapher;
begin
  EnCurve := TXYGrapher.Create;
  with EnCurve do
  begin
    SetFormSize(550, 500);
    j := layer;
    with Soil do
    for i := 0 to 50 do
    begin
      GX[i] := 0.02 * i + 0.001;
      GY[i] := Aen[j]  + Ben[j] * GX[i] - (Aen[j] - 1) * exp(-Cen[j] * Power(GX[i], Een[j]));
    end;
  dumstr := ExeDir + Slash + 'SoilProperties' + Slash + SoilName[layer];
  if DirectoryExists(dumstr) = False then CreateDir(dumstr);
  AssignFile(f, dumstr + Slash + SoilName[layer] + '_en.txt');
    Rewrite(f);
    Writeln(f,'Enhancement factor of ' + SoilName[j]);
    Writeln(f, '  Akh       Bkh      Ckh      Dkh      Ekh');
    with Soil do
      Writeln(f, ' ' + FloatToStr(Akh[j]) + '    ' + FloatToStr(Bkh[j]) + '    ' + FloatToStr(Ckh[j]) + '    ' + FloatToStr(Dkh[j]) + '    ' + FloatToStr(Ekh[j]));
    Writeln(f, '  V.W.C.   Enhancement factor');
    for i := 0 to 50 do
      Writeln(f, '  ' + FormatFloat('0.0000', GX[i]) + '      ' + FormatFloat('0.00000', GY[i]));
    CloseFile(f);
    //---------------------------------
    NumDataSeries := 1;
    //----------
    DataSeries[1] := dumstr + Slash + SoilName[layer] + '_en.txt';
    Xb[1] := 1; Xt[1] := 0; Yl[1] := 2; Yr[1] := 0; ReadStart[1] := 6; ReadEnd[1] := 0;
    LineStyle[1] := 1; LineWidth[1] := 0.25; LineColor[1] := 33023; MarkStyle[1] := 0; MarkSize[1] := 0;
    Xmult[1] := 1; Xoffset[1] := 0; Ymult[1] := 1; Yoffset[1] := 0;
    //----------
    Height := 65; Width := 65; Lmargin := 30; Tmargin := 20; FLineWidth := 0.4; NumFontSize := 9;
    NumFontName := 'Arial';
    Max[1] := 1 ; Min[1] := 0 ; Division[1] := 5; Minordivision[1] := 2; logarithm[1] := 0; NumeralsXshift[1] := 0; NumeralsYshift[1] := 0; ZeroLine[1] := 0;
    Max[2] := 1 ; Min[2] := 0 ; Division[2] := 5; Minordivision[2] := 2; logarithm[2] := 0; NumeralsXshift[2] := 0; NumeralsYshift[2] := 0; ZeroLine[2] := 0;
    Max[3] := 25; Min[3] := 0; Division[3] := 5; Minordivision[3] := 0; logarithm[3] := 0; NumeralsXshift[3] := 0; NumeralsYshift[3] := 0; ZeroLine[3] := 0;
    Max[4] := 25; Min[4] := 0; Division[4] := 5; Minordivision[4] := 0; logarithm[4] := 0; NumeralsXshift[4] := 0; NumeralsYshift[4] := 0; ZeroLine[4] := 0;
    NumLegend := 5;
    for i := 2 to 5 do
      Legend[i] := Msg[i+32];
    if LANG = 128 then
      Legend[1] := SoilName[j] + Msg[33]
    else
      Legend[1] := Msg[33] + SoilName[j] + '\n';
    LegendLspacing[1] := 4; LegendXshift[1] := 0; LegendYshift[1] := 0;
    LegendLspacing[2] := 0; LegendXshift[2] := 0; LegendYshift[2] := 0;
    LegendLspacing[3] := 0; LegendXshift[3] := 0; LegendYshift[3] := 0;
    LegendLspacing[4] := 0; LegendXshift[4] := 0; LegendYshift[4] := 0;
    LegendLspacing[5] := 0; LegendXshift[5] := 0; LegendYshift[5] := 0;
    SaveFigFile(FileName);
    ExecuteXYGrapher(FileName);
  end;
  EnCurve.Free;
end;

procedure TOutputResults.SliceRings();
var
  f: TextFile;
  i, i2, j, m: Integer;
  Borlder: array[0..100] of Double;
  dz_: Double;
  function GetWaterContent(z_: Double): Double;
  var i: Integer;
  begin
    if z_ < z[1] then
      i := 2
    else
      if z_ > z[n] then
        i := n
      else
      begin
        while z_ > z[j] do
          j := j + 1;
        i := j;
      end;
    Result := (W[i] - W[i-1]) / (z[i] - z[i-1]) * (z_ - z[i-1]) + W[i-1];
  end;
begin
  AssignFile(f, Main.GetAbsoluteDirectory(BaseDir, 'BorderDepths.txt'));
  Reset(f);
  i := 0;
  j := 1;
  Borlder[0] := 0;
  repeat
    i := i + 1;
    Read(f, Borlder[i]);
  until EOF(f);
  CloseFile(f);
  i2 := i - 1;
  AssignFile(f, Main.GetAbsoluteDirectory(BaseDir, CaseName + '_numerically_sliced_water_content_profile.out'));
  Rewrite(f);
  for i := 1 to i2 do
  begin
    dum := 0;
    dz_ := (Borlder[i] - Borlder[i-1]) / 10;
    for m := 1 to 9 do
      dum := dum + GetWaterContent(Borlder[i-1] + dz_ * m);
    dum := dz_ / 2 * (GetWaterContent(Borlder[i-1]) + 2 * dum + GetWaterContent(Borlder[i]));
    dum := dum / (Borlder[i] - Borlder[i-1]);
    Writeln(f, FormatFloat('#0.00', Borlder[i]) + '  ' + FormatFloat('0.000', dum));
  end;
  CloseFile(f);
end;

end.
