{
_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/

        WASH_1D version 1.36

        Copyright(C) 2000-2004 Haruyuki FUJIMAKI and Mitsuhiro INOUE
        fujimaki@sakura.cc.tsukuba.ac.jp
        URL: http://www.geocities.jp/fujimaki691202/
        Distributed under the GPL

_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
}

unit Simulation;

interface

uses
{$ifdef Linux}
  SysUtils, Classes, QGraphics, QControls, QForms, 
  QDialogs, Math, QExtCtrls, QStdCtrls, QComCtrls;
{$else}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Math,
  ExtCtrls, StdCtrls, {$WARN UNIT_PLATFORM OFF}FileCtrl{$WARN UNIT_PLATFORM ON}, ComCtrls;
{$endif}

const
  ETAs = 0.6; ETAh = 0.6; Margin = 0.01; NumElem = 801;

type
  TValues = array[0..NumElem] of Double;
  TSimulator = class(TObject)
    function GetMilliTime:Double;
  private
    procedure Observate;
    procedure CalculateVaporMovement(ToEnd: Integer);
    procedure CalculateSoluteMovement;
    procedure CalculateHeatMovement;
    //procedure Report;
    procedure SolveTheCoefficientMatrix;
    procedure SetScanningCurve(j, i: Integer; Drying_: Boolean);
    procedure NumericalExperiment;
    function GetOneStepDisp(i: Integer): Double;
    function CalculateAtmos_BC: Boolean;
    function CalculateLiquidFlow: Boolean;
  public
    constructor Create; //override;
    function SetLengthRJ(dumstr: String; dumint: Shortint): String;
    function GetResidual(x: Double; n: Integer): Double;
    function GetScaledParameter(x: Double; i, j: Integer; Drying_: Boolean): Double;
    function GetAvgWaterContentAbove(Depth: Double): Double;
    procedure Simulate;
    procedure DrawFigure;
    procedure DetermineSpaceIncrements;
    procedure OutputToFile;
  end;

var
  Simulator: TSimulator;
  ETA, MaxEvap, t_conv, dt_min: Double;
  z_c : TValues;
  Ath, Bth, Cth, Dth, U, x: TValues;
  W, Wb, Wtent, Cw, Sw, P, Pb, Pold, K                    : TValues;
  ql, qv, Cv, Cvsat, RH, EN, Dv, rho                      : TValues;
  C, M, Crystl, Di, Dm, Dc, qs, Wold, Wbold, Wolds, Ss, pi, V     : TValues;
  Chnew, kh, qh, LH, Sh                                   : TValues;
  T, Tb, Ch, h                                            : TValues;
  Drying                                                  : array[0..NumElem] of Boolean;
  Wsat_A, zeta_A, Wr_A                                   : TValues; // Adapted
  TargetMaxDiff_logP, TargetESRErrorW, CriteriaForCeliasIteration, Gravity: Double;
  LowerTargetIteration, UpperTargetIteration, IterationScheme, InterHyCon : ShortInt;
  SL: TStringList;
  iteration: Integer;
  CanceledTime, MaxDiff_logP: Double;
  Canceled: Boolean;
  // for fingering study
  MLB: Boolean = False;//True;

implementation

uses WASH1dMAIN, RC_Params, K_Params, RootFinder;

constructor TSimulator.Create;
begin
  inherited Create;
  TargetMaxDiff_logP  := 1;
  TargetESRErrorW := 0.0001;
  CriteriaForCeliasIteration := 0.0001;
  LowerTargetIteration := 3;
  UpperTargetIteration := 7;
  IterationScheme := 2;
  InterHyCon := 1;
  SL := TStringList.Create;
end;

procedure TSimulator.Simulate;
var
  f: TextFile;
  i, i2, j: Integer;
  starttime: Double;
  SL1, SL2: TStringList;
  procedure LoadMeasuredProfile(FileName: String; var Y: TValues);
  var
    zm, Ym: TValues;
    i, i2, i3: Integer;
  begin
    SL1 := TStringList.Create;
    SL2 := TStringList.Create;
    i2 := 1;
    try
      SL1.LoadFromFile(FileName);
      for i := 1 to SL1.Count do
      begin
        SL2.CommaText := SL1[i-1];
        try
          zm[i2] := StrToFloat(SL2[0]);
          Ym[i2] := StrToFloat(SL2[1]);
          i2 := i2 + 1;
        except
        end;
      end;
    finally
      SL1.Free;
      SL2.Free;
    end;
    i3 := i2;
    for i := 1 to n + 1 do
    begin
      i2 := 1;
      while (z[i] >= zm[i2]) and (i2 < i3) do
        i2 := i2 + 1;
      if z_c[i] < zm[1] then
        Y[i] := ((Ym[2] - Ym[1]) / (zm[2] - zm[1])) * (z_c[i] - zm[1]) + Ym[1]
      else
        if z_c[i] > zm[i3-1] then
          Y[i] := ((Ym[i3-1] - Ym[i3-2]) / (zm[i3-1] - zm[i3-2])) * (z_c[i] - zm[i3-2]) + Ym[i3-2]
        else
          Y[i] := (Ym[i2-1] * (zm[i2] - z_c[i]) + Ym[i2] * (z_c[i] - zm[i2-1])) / (zm[i2] - zm[i2-1]);
    end;
  end;
begin
  DetermineSpaceIncrements;
  //i := n+2;
  //SetLength(W,i)   ; SetLength(Wb,i)   ; SetLength(Wtent,i)  ; SetLength(Cw,i)  ; SetLength(Sw,i);
  //SetLength(P,i)   ; SetLength(Ptent,i); SetLength(Pbefore,i);
  //SetLength(K,i)   ; SetLength(Kmid,i) ; SetLength(ql,i)     ; SetLength(qv,i)  ;
  //SetLength(Cv,i)  ; SetLength(Cvsat,i); SetLength(RH,i)     ; SetLength(EN,i)  ; SetLength(Dv,i);
  //SetLength(C,i);
  //SetLength(T,i)   ; SetLength(Tb,i)   ; SetLength(Ch,i)     ; SetLength(H,i);
  //SetLength(THA,i) ; SetLength(THB,i)  ; SetLength(THC,i)    ; SetLength(THD,i) ; SetLength(U, i);
  //SetLength(THBD,i); SetLength(Dthd,i) ; SetLength(x,i);
  //i := Trunc(t_end / int_rec) + 1;
  //SetLength(time,i)  ; SetLength(Wtime,i); SetLength(Ptime,i)  ; SetLength(Ttime,i); SetLength(CumE,i); SetLength(CumD,i);
  //SetLength(ql_bot,i); SetLength(Evap,i) ;   //ưŪˤint_recȤǿϾʤˤ⤫餺Access Violation ErrorǤ롣
//------------------- Initialization -----------------//
  StopCalc := 0;
  time_CRWM := 0; dt_CRWM_next := 20 * dt_ini;
  time_ := 0; dt := dt_ini;
  abc_done  := 0; Pbot_done := 1; OutFlow_done := 1; qltop_done := 1;
  CumEvap  := 0; CumInfilt := 0; CumDrain := 0; counter := 0; NumCalc := 0;
  Cum_qs_top := 0;  Cum_qs_bot := 0; Crystal := 0; V1before  := 0; Vnbefore := 0;
  CumSHT := 0; CumLHT :=  0; CumCHL :=  0; CumHLB :=  0; CumHLS :=  0;
  Gravity := cos(3.1415926535897932385 * Angle / 180.0);
  CanceledTime := 0;
  t_conv := 86400 / t_unit;
  dt_min := 0.01 / t_unit;
  if IterationScheme < 2 then
    ETA := 0.6
  else
    ETA := 1;
  if Tair = 0 then
    Tair := Soil.Tref[1];
  Cv[1] :=  exp(6 - 4976 / Tair);
  if Mode <> 2 then
    IPET_type := 0;
  if NumExp = 1 then
    t_evaporation[1] := 100000000000000;
  if IPET_type = 2 then
    ra := 999;
  for j := 1 to NumObs do
    for i := 1 to Ceil(t_end / int_rec) do
      Ptime[j,i] := 0;
  for i := 1 to 3000 do
    Front[i] := 0;
  //============ Constants =============//
  if abc_type  = 2 then
    Cva := RHair * exp(6 - 4976 / Tair);
  case s_type of
  1:begin
    Diw0   := 0.000015 * t_unit;
    Cmax   := 310;
    NumIon := 2;
    Mweight :=  59;
    a_oc   := 0.0026;
    b_oc   := -0.025;
    c_oc   := 0.53;
    end;
  2:begin
    Diw0 := 0.000015 * t_unit;
    Cmax := 310;
    NumIon := 2;
    Mweight :=  59;
    a_oc   := 0.0026;
    b_oc   := -0.025;
    c_oc   := 0.53;
    end;
  end;
//===================  Initial Condition  ========================
  //----- water -----//
  for i := 1 to n+1 do
  begin
    qv[i] := 0;
    ql[i] := 0;
  end;
  with Soil do
    if (Water = 1) or (IPET_type = 11) then
    begin
      {
      for j := 1 to NumLayer do
        if RF[j] = 1 then
          LogOfLlambda[j] := ln(Llambda[j]);
      }
      for i := 1 to n + 1 do
      begin
        if IniDrying then
          Drying[i] := True
        else
          Drying[i] := False;
        j := L[i];
        Wsat_A[i] := Wsat[j];
        zeta_A[i] := zeta[j];
        Wr_A[i]   := Wr[j];
       // Just_a_moment[i] := 0;
      end;
      if heat = 0 then
        Tini := Tair;
      case ic_type of
      1:for i := 1 to n do
        begin
          j := L[i];
          p[i] := Pressure_Head(w_ini, Tini, Wsat_A[i], zeta_A[i], Wr_A[i], j, Drying[i]);
          W[i] := W_ini;
        end;
      2:for i := 1 to n + 1 do
        begin
          j := L[i];
          p[i] := p_ini;
          W[i] := Water_Content(p[i], Tini, Wsat_A[i], zeta_A[i], Wr_A[i], j, Drying[i]);
        end;
      3:for i := 1 to n + 1 do
        begin
          j := L[i];
          p[i] := p_top_ini + (z[i] + z[i-1]) / 2;
          W[i] := Water_Content(p[i], Tini, Wsat_A[i], zeta_A[i], Wr_A[i], j, Drying[i]);
        end;
      4:begin
          LoadMeasuredProfile(fl_profile, W);
          for i := 1 to n + 1 do
          begin
            j := L[i];
            P[i] := Pressure_Head(W[i], Tini, Wsat_A[i], zeta_A[i], Wr_A[i], j, Drying[i]);
            if P[i] > -0.2 then
              W[i] := Wsat_A[j];
          end;
        end;
      end;
    end
    else
      for i := 1 to n + 1 do
      begin
        p[i]   := 0.0;
        W[i]   := 0.3;
      end;
  if Hpond > 0 then
  begin
    P[1] := Hpond;
    W[1] := Soil.Wsat[1] + Soil.Cwsat * Hpond;
  end;
  W_storage_ini  := 0;
  for i := 1 to n do
    W_storage_ini := W_storage_ini + W[i] * dz[i];
  W_storage_old := W_storage_ini;
  W[n+1] := W[n]; W[0] := W[1];  // imaginary values.
  for i := 1 to n + 1 do
  begin
    Pold[i] := P[i];
   // Pb[i] := exp((ln(Abs(P[i])) * dz[i+1] / 2 + ln(Abs(P[i+1])) * dz[i] / 2) / ((dz[i] + dz[i+1]) / 2));
    Wtent[i] := W[i];
    Wold[i]  := W[i];
    Wolds[i] := W[i];
    Wb[i] := (W[i] * dz[i+1] / 2 + W[i+1] * dz[i] / 2) / ((dz[i] + dz[i+1]) / 2);
    Wbold[i] := Wb[i];
  end;
               // _b : at the compartment boundaries --
  Wb[n+1] := Wb[n];
  //----- heat -----//
  if heat=0 then
    for i := 1 to n + 1 do T[i] := Tair
  else
    for i := 1 to n + 1 do T[i] := Tini;
  E_Storage_ini := 0;
  for i := 1 to n do
  begin
    j := L[i];
    Ch[i] := (2.13 * (1 - Soil.Wsat[j]) + 4.18 * W[i]);
    h[i] := Ch[i] * T[i];
    E_Storage_ini := E_Storage_ini + h[i] * dz[i];
  end;
  if Tbot <> 0 then
    T[n+1] := Tbot
  else
    T[n+1] := Tini;
  for i := 1 to n do
    Tb[i] := (T[i] * dz[i+1] / 2 + T[i+1] * dz[i] / 2) / ((dz[i] + dz[i+1]) / 2);
  E_Storage_old := E_Storage_ini;
  //------- solute -------//
  if solute <>1 then
    C_ini := 0;
  S_Storage_ini := 0;
  case s_ic_type of
  1:for i := 1 to n do
    begin
      C[i] := C_ini;
      M[i] := C[i] * W[i];
      Crystl[i] := 0;
    end;
  2:begin
      LoadMeasuredProfile(fl_iniCprofile, C);
      for i := 1 to n do
      begin
        M[i] := C_ini * W[i];
        Crystl[i] := 0;
      end;
    end;
  3:begin
      LoadMeasuredProfile(fl_iniCprofile, M);
      for i := 1 to n do
        if M[i] / W[i] < Cmax then
          C[i] := M[i] / W[i]
        else
        begin
          C[i] := Cmax;
          Crystl[i] := M[i] - W[i] * Cmax;
        end;
    end;
  end;
  for i := 1 to n do
    S_Storage_ini := S_Storage_ini + M[i] * dz[i];
  C[n+1] := C[n];
  if CRWM = 1 then
  begin
    for i := 1 to Trunc(Crystal / MPP) do
      zp[i] := -1;
    NumP := Trunc(Crystal / MPP);
    for j := 1 to n do
    begin
      for i := NumP + 1 to NumP + Trunc(M[j]*dz[j]/ MPP) do
      begin
        zp[i]  := z[j-1] + dz[j] * Random;
        ncp[i] := j;
      end;
      NumP := Trunc(M[j]*dz[j] / MPP) + NumP;
    end;
  end;
  if water <> 1 then
  begin
    if Mode = 1 then
      for i := 0 to n + 1 do
        ql[i] := 0.3 * Vfixed
    else
      for i := 0 to n + 1 do
        ql[i] := ql_top;
  end;
  //----------------------------------------------------------
  Observate;
  if graph = 1 then
  begin
    Main.Profile;
    Main.ProfileDraw(W,z,n,0,0.6,clBlue);
    Main.ProfileDraw(C,z,n,0,6,$009090);
    Main.ProfileDraw(T,z,n,273.2,333.2,clRed);
  end;
  //================= Load of Boundary Condition Data ================//
  //--- Upper Boundary Flux ---//
  if u_bc_type = 3 then
  begin
    SL1 := TStringList.Create;
    SL2 := TStringList.Create;
    i2 := 0;
    try
      SL1.LoadFromFile(fl_ql_top);
      for i := 1 to SL1.Count do
        begin
        SL2.CommaText := SL1[i-1];
        try
          t_ql_top[i2] := StrToFloat(SL2[0]);
          qltop[i2]    := StrToFloat(SL2[1]);
          i2 := i2 + 1;
        except
        end;
        end;
    finally
      SL1.Free;
      SL2.Free;
    end;
    Num_qltop_Data := i2 - 1;
  end;
  //---- Atmospheric Boundary Condition ----//
  case abc_type of
  3:begin
      SL1 := TStringList.Create;
      SL2 := TStringList.Create;
      i2 := 1;
      try
        SL1.LoadFromFile(fl_CumEvap);
        for i := 1 to SL1.Count do
        begin
          SL2.CommaText := SL1[i-1];
          try
            flx[i2] := StrToFloat(SL2[0]);
            fly[i2] := StrToFloat(SL2[1]);
            i2 := i2 + 1;
          except
          end;
        end;
      finally
        SL1.Free;
        SL2.Free;
      end;
      dumint := i2;
      for i := 1 to dumint do
      begin
        t_clm_con[i]      := flx[i];
        CumEvaporation[i] := fly[i];
      end;
      t_clm_con[dumint + 1] :=  100000000000000000.0;
      CumEvaporation[dumint + 1]  := CumEvaporation[dumint-1];
   end;
  4:begin
      SL1 := TStringList.Create;
      SL2 := TStringList.Create;
      i2 := 0;
      try
        SL1.LoadFromFile(fl_clmtc_con);
        for i := 1 to SL1.Count do
        begin
          SL2.CommaText := SL1[i-1];
          try
            t_clm_con[i2] := StrToFloat(SL2[0]);
            WV[i2]        := StrToFloat(SL2[1]);
            Ta[i2]        := StrToFloat(SL2[2]) + 273.2;
            RHa[i2]       := StrToFloat(SL2[3]) / 100;
            CPR[i2]        := StrToFloat(SL2[4]) / 10;   // mm         --> cm
            CRg[i2]        := StrToFloat(SL2[5]) * 100;  // MJ/m2  --> J/cm2
            i2 := i2 + 1;
          except
          end;
        end;
      finally
        SL1.Free;
        SL2.Free;
      end;
    end;
  end;
  if s_u_bc_type = 2 then
  begin
    SL1 := TStringList.Create;
    SL2 := TStringList.Create;
    i2 := 0;
    try
      SL1.LoadFromFile(fl_C_irg);
      for i := 1 to SL1.Count do
      begin
        SL2.CommaText := SL1[i-1];
        try
          t_c_irg[i2] := StrToFloat(SL2[0]);
          Cirg[i2]    := StrToFloat(SL2[1]);
          i2 := i2 + 1;
        except
        end;
      end;
    finally
      SL1.Free;
      SL2.Free;
    end;
    NumCirgData := i2 - 1;
  end;
  //----- Lower Boundary Data ----//
  if Mode = 1 then
    case l_bc_type of
    4:begin
      SL1 := TStringList.Create;
      SL2 := TStringList.Create;
      i2 := 0;
      try
        SL1.LoadFromFile(fl_p_bot);
        for i := 1 to SL1.Count do
        begin
          SL2.CommaText := SL1[i-1];
          try
            t_p_bot[i2] := StrToFloat(SL2[0]);
            Pbot[i2]    := StrToFloat(SL2[1]);
            i2 := i2 + 1;
          except
          end;
        end;
      finally
        SL1.Free;
        SL2.Free;
      end;
      NumPbotData := i2 - 1;
      end;
    5:begin
      SL1 := TStringList.Create;
      SL2 := TStringList.Create;
      i2 := 1;
      try
        SL1.LoadFromFile(fl_outflow);
        for i := 1 to SL1.Count do
        begin
          SL2.CommaText := SL1[i-1];
          try
            if StrToFloat(SL2[0]) < t_outflow[i2-1] then Break;
            t_outflow[i2] := StrToFloat(SL2[0]);
            CumOutFlow[i2] :=  StrToFloat(SL2[1]);
            i2 := i2 + 1;
          except
          end;
        end;
      finally
        SL1.Free;
        SL2.Free;
      end;
      NumOutflowData_for_DP := i2 - 1;
      end;
    end;
  if s_l_bc_type = 2 then
  begin
    SL1 := TStringList.Create;
    SL2 := TStringList.Create;
    i2 := 0;
    try
      SL1.LoadFromFile(fl_C_bot);
      for i := 1 to SL1.Count do
      begin
        SL2.CommaText := SL1[i-1];
        try
          t_c_bot[i2] := StrToFloat(SL2[0]);
          Cbot[i2]    := StrToFloat(SL2[1]);
          i2 := i2 + 1;
        except
        end;
      end;
    finally
      SL1.Free;
      SL2.Free;
    end;
    NumCbotData := i2 - 1;
  end;
  if (CRWM = 1) and (water = 0) then
    Soil.dispersivity[1] := Ds / Vfixed;
  //-- Other Preparation --//
  Main.ProgressBar1.Max := Trunc(t_end / int_rec);
  Main.ProgressBar1.Position := 0;
  //Main.ProgressBar1.Top := 1 + Main.Panel4.Top + Main.Panel4.Height;
  SL := TStringList.Create;
  with SL do
  begin
    Clear;
    Add('Time evolution of pressure head at the bottom');
    Add('   time     pressure head(cm)');
    Add(SetLengthRJ('0', 12) + '  ' + FormatFloat('###0', P_bot));
  end;
  {$I-}
  if graph <> 1 then
    Main.ProgressBar1.Visible := True;
  starttime := GetMilliTime;
///////////////////////////////  Main Loop  //////////////////////////////////
  repeat
    //--------  Numarical Analysys  ------------//
    if water  = 1 then
    begin
      if CalculateAtmos_BC = False then
      begin
        DrawFigure;
        Break;
      end;
      CalculateVaporMovement(n);
      if CalculateLiquidFlow = False then
      begin
        DrawFigure;
        Break;
      end;
    end;
    if Canceled = False then
    begin
      if solute = 1 then
        CalculateSoluteMovement;
      if heat   = 1 then
        CalculateHeatMovement;
      time_ := time_ + dt;
    end;
    //--------  Record and Graphical Output ---------//
    if time_ >= int_rec * (counter + 1) then
    begin
      counter := counter + 1;
      if CRWM = 1 then
      begin
        if water <> 1 then
          for i := 0 to n + 1 do
            ql[i] := 0.3 * Vfixed;
        dumint := 12345;
        CalculateSoluteMovement;
      end;
      Observate;
      if NumExp = 1 then
        NumericalExperiment;
      if (graph = 1) and (Mode = 1) then
        DrawFigure
      else
        Main.ProgressBar1.StepIt;
    end;
    if StopCalc = 1 then Exit;
    //--------- Time Step Control ------------//
    avdt := avdt + dt; navdt := navdt + 1;
    //-- water --//
    if water = 1 then
    begin
      if IterationScheme = 2 then
      begin
        if iteration <= LowerTargetIteration then
          dt := dt * 1.3
        else
          if iteration >= UpperTargetIteration then
            dt := dt * 0.7;
      end
      else
      begin
        if ESRErrorW > TargetESRErrorW then
          dt := dt * 0.7
        else
          dt := dt * 1.3;
      end;
      if MaxDiff_logP > TargetMaxDiff_logP then
        dt := dt * 0.5;
      if dt > dt_max then
        dt := dt_max;
      if dt < dt_min then
        dt := dt_min;
    end
    else
      dt := dt * 1.2;
    //-- solute --//
    if (solute = 1) and (CRWM <> 1) and (time_ > 0) then
    begin
      for i := 1 to n - 1 do
      begin
        dum := 2 * dz[i] * dz[i] / Dc[i] / 2;
        if dum < dt then
          dt := dum;
      end;
      dt := dt;
    end;
    if time_ + dt > int_rec * (counter + 1) then
      dt := int_rec * (counter + 1) - time_ + 0.000001;
  until time_ >= t_end;
//////////////////////////////////////////////////////////////////////////////
  DrawFigure;
  Observate;
  if NumExp = 1 then
  begin
    SL.Add(SetLengthRJ(FormatFloat('#######0.0000', time_), 12) +'  '+ FormatFloat('###0', P_bot));
    SL.SaveToFile(caseName+'_Pbot.dat');
  end;
  SL.Free;
  Main.ProgressBar1.Visible := False;
  if Mode = 1 then
  begin
    Beep;
    ShowMessage('Run time = '+ FormatFloat('####0.00',GetMilliTime - starttime) + ' sec');
   // ShowMessage('Canceled time = ' + FloatToStr(CanceledTime));
  end
  else
    ;//ShowMessage('ak ='+ FormatFloat('#0.00',ak)+' bk = '+ FormatFloat('#0.00',bk));
  if AllowRecord = 1 then
    OutputToFile;
end;
//----------------------------------------------------------

{ for numerical experiment of Two-Stage Method or Multi-Step Outflow Method}
procedure TSimulator.NumericalExperiment;
  procedure WriteOnFile;
  begin
    SL.Add(SetLengthRJ(FormatFloat('#######0.0000', time_), 12) + '  '+ FormatFloat('###0', P_bot));
  end;
begin
  if (Abs(ql[n]*10*86400 / t_unit) < ChgCriteria)
    and (qv[0] > -0.00000000000000001) then
    if MultiStep < 2 then
    begin
      if P[4] < P_bot / 2 then
      begin
        if Abs(P_bot - LowestPbot) > 0.1 then
          WriteOnFile;
        if Soil.Ksat[1] < 0.01 then
          if P_bot > LowestPbot / 2 - 0.1 then
          begin
            P_bot := 4 * P_bot;
            WriteOnFile;
          end
          else
            if MultiStep = 0 then
            begin
              t_evaporation[1] := time_;
              WriteOnFile;
            end
        else
          if P_bot > -35.1 then
          begin
            P_bot := P_bot - 5;
            WriteOnFile;
          end
          else
          begin
            t_evaporation[1] := time_;
            WriteOnFile;
          end;
      end
    end
    else
      if P[4] < P_bot / 2 then
      begin
        if Abs(P_bot - LowestPbot) < 0.1 then
        begin
          WriteOnFile;
          t_end := time_ * 0.99;
          Application.ProcessMessages;
        end;
        if P_bot < LowestPbot / 2 - 0.1 then
        begin
          P_bot := 0.5 * P_bot;
          WriteOnFile;
        end;

      end;
end;

procedure TSimulator.DrawFigure;
var
  i, GYmax, GYmin: Integer;
begin
  Main.Profile;
  Main.ProfileDraw(W, z_c, n, 0, 0.6, clBlue);
  if solute = 1 then
  begin
    Main.ProfileDraw(C, z_c, n, 0, 6, $009090);
    if Mode = 2 then
      Main.ProfileDraw(Cm, zms, NumSampling, 0, 6, clRed);
  end;
  if heat = 1 then
    Main.ProfileDraw(T, z_c, n, 273.2, 333.2, $000080FF)
  else
  begin
    if P[n] < -150 then
      dum := -900
    else
      dum := -150;
    Main.ProfileDraw(P,z_c, n, dum, 0,clGray);
  end;
  if (Mode = 2) and ((IPET_type = 1) or (IPET_type = 5)) then
    Main.ProfileDraw(Wm, zm, NumSampling, 0, 0.6, clOlive);
  for i := 0 to n do
  begin
    ql[i] := -ql[i]*10*86400/ t_unit;
    qv[i] := -qv[i]*10*86400/ t_unit;
  end;
  Main.History;
  if Mode = 1 then
  begin
    Main.FluxProfile;
    Main.FluxProfileDraw(ql,z_c, clNavy);
    Main.FluxProfileDraw(qv,z_c, clAqua);
    for i := 1 to n-1 do
    begin
      ql[i] := -ql[i] * t_unit / (10 * 86400);
      qv[i] := -qv[i] * t_unit / (10 * 86400);
    end;
  end
  else
    Main.PropertyDraw;
  //counter := counter - 1;
  for i := 1 to counter do
    dumarray[i] := Wtime[1, i];
  Main.HistoryDraw(time, dumarray, 1, counter, -0.6, 0.4, clBlue, psSolid);
  if solute = 1 then
  begin
    for i := 1 to counter do
      dumarray[i] := Ctime[1, i];
    Main.HistoryDraw(time, dumarray, 1, counter, -12 ,   8, $009090, psSolid);
  end;
  if heat = 1 then
  begin
    for i := 1 to counter do
      dumarray[i] := Ttime[1, i];
    Main.HistoryDraw(time, dumarray, 1, counter, -60 ,  40, $000080FF, psSolid);
  end;
  if Mode = 2 then
  begin
    if (IPET_type = 2) and (UseOutFlowForK = 0) and (l_bc_type = 4) and (Tensio > 0) and (z_lb0 > 0) then
      dum := z_lb / z_lb0
    else
      dum := 1;
    for i := 1 to NumOutflowData do
      dumarray[i] := dum * CumOutflow[i];
    case IPET_type of
    5:begin
        GYmin := -12;
        GYmax :=   8;
      end;
    6:begin
        GYmin := -30;
        GYmax :=  20;
      end;
    else
      begin
        GYmin := -3;
        GYmax :=  2;
      end;
    end;
    Main.HistoryDraw(t_outflow, dumarray, 1, NumOutflowData, GYmin, GYmax, clOlive, psSolid);
    Main.HistoryDraw(time, CumD, 1, counter, GYmin, GYmax, clNavy, psSolid);
    if (IPET_type = 1) or (IPET_type = 4) then
      Main.HistoryDraw(t_evaporation, CumEvaporation, 1, NumEvapData, -3 , 2, clOlive, psSolid);
    if (IPET_type = 2) or (IPET_type = 4) or (IPET_type = 6) then
      Main.HistoryDraw(t_tensio, Ptens1, 1, NumTensioData, -600, 400, clOlive, psSolid);
    if (IPET_type <> 1) or (UseOutFlowForK = 1) then
      Main.HistoryDraw(t_P_bot, Pbot, 0, NumPbotData, -600, 400, clTeal, psDot);
    Main.HistoryDraw(time, CumE, 1, counter, -3 , 2, clAqua, psSolid);
  end
  else
  begin
    Main.HistoryDraw(time, ql_bot, 1, counter, -60, 40, clNavy, psSolid);
    Main.HistoryDraw(time, Evap  , 1, counter, -60, 40, clAqua, psSolid);
  end;
  for i := 1 to counter do
    dumarray[i] := Ptime[1, i];
  Main.HistoryDraw(time, dumarray, 1, counter, -600, 400, clTeal, psSolid);
  //counter := counter + 1;
  Main.Report;
  qv[0]  := -qv[0]/(10*86400) * t_unit;
  ql[0]  := -ql[0]/(10*86400) * t_unit;
  ql[n]  := -ql[n]/(10*86400) * t_unit;
  avdt := 0; navdt := 0;
  Application.ProcessMessages;
end;

procedure TSimulator.OutputToFile;
var
  i, j: Integer;
  SL1, SL2: TStringList;
begin
  AssignFile(f, CaseName + '_profile.out');
  Rewrite(f);
  Writeln(f, '>>>>>>>>>>>>>>>>> Result of the Simulation <<<<<<<<<<<<<<<<<');
  Writeln(f, '    time       counter  number of calculation');
  Writeln(f, '   '+ FormatFloat('#######0.000', time_)+'     '+IntToStr(counter)+'       '+ IntToStr(NumCalc));
  Writeln(f, ' W_storage_ini   CumDrain    CumInfilt   CumEvap   W_storage');
  Writeln(f, '   '+ FormatFloat( '#0.0000',W_storage_ini)+'       '+ FormatFloat('##0.0000',CumDrain)+'       '+
             FormatFloat('##0.0000',CumInfilt)+  '       '+ FormatFloat('##0.0000',CumEvap)+'       '+ FormatFloat('#0.0000',W_storage_old));
  if solute = 1 then
  begin
    Writeln(f, ' S_storage_ini   Cum_qs_bot  Cum_qs_top  Crystal  S_storage');
    Writeln(f, '   ' + FormatFloat( '#0.0000',S_storage_ini) + ',      '+ FormatFloat('##0.00',Cum_qs_bot)+'       '+
             FormatFloat('##0.00',Cum_qs_top)+ '       '+ FormatFloat('##0.00', Crystal)+'       '+ FormatFloat('#0.00',S_storage));
  end;
  Writeln(f, 'z(cm), VWC, T(C), c(mg/cm3), VSC(mg/cm3)');
  for i := 1 to n do
  begin
    if (z[i] + z[i-1]) / 2 < 10 then
      dumstr := ' '
    else
      dumstr := '';
    Writeln(f, dumstr + FormatFloat('#0.00#', (z[i] + z[i-1]) / 2) + ',  ' +  FormatFloat('0.0000', W[i]) + ',  ' +
            FormatFloat('#0.0',T[i]-273.2) + ', ' + SetLengthRJ(FormatFloat('##0.00', C[i]),5) + ', '+ SetLengthRJ(FormatFloat('##0.00', M[i]), 6)) ;// ql[i] * 864000; qv[i] * 864000: end;
  end;
  CloseFile(f);
//-------------------
  AssignFile(f, caseName + '_flux.out');
  Rewrite(f);
  Writeln(f, '>>>>>>>>>>>> Cumulative Evaporation and Outflow <<<<<<<<<<<<<');
  Writeln(f, '     time        CumE       CumD       CumI       Front');
  for i := 1 to counter do
    Writeln(f, SetLengthRJ(FormatFloat('######0.000', i * int_rec), 11) +'   '+
        SetLengthRJ(FormatFloat('#0.0000', CumE[i]),8)+'   '+
        SetLengthRJ(FormatFloat('#0.0000', CumD[i]),8)+'   '+
        SetLengthRJ(FormatFloat('#0.0000', CumI[i]),8)+'   '+
        SetLengthRJ(FormatFloat('#0.0000', Front[i]),8));
  CloseFile(f);
  for j := 1 to NumObs do
  begin
    AssignFile(f, caseName + '_observation' + IntToStr(j) + '.out');
    Rewrite(f);
    Writeln(f, '>>>>>>>>>>>> Values at the Observation Point <<<<<<<<<<<<<');
    Writeln(f, 'depth = ' + FormatFloat('##0.0', z_obs[j]) + 'cm');
    Writeln(f, 'time, VWC, pressure(cm), c(mg/cm3), T(C), avgVWC_above_the_depth');
    for i := 1 to counter do
      Writeln(f, FormatFloat('######0.000', i * int_rec) + '     ' +
                 FormatFloat('0.0000', Wtime[j,i])+ '     ' +
                 FormatFloat('#######0.0', Ptime[j,i]) + '     ' +
                 FormatFloat('##0.0000', Ctime[j,i]) + '     ' +
                 FormatFloat('#0.0', Ttime[j,i]) + '     '  +
                 FormatFloat('0.000', AvgWaterContentAbove[j,i]));
    CloseFile(f);
  end;
  SL1 := TStringList.Create;
  SL2 := TStringList.Create;
  try
    if (NumExp = 1) and (Mode = 1) then
      begin
      with SL1 do
        begin
        Clear;
        Add('Hypothetical outflow data');
        Add('   time    CumOutflow');
        i := 0;
        repeat
          i := i + 1;
          Add(SetLengthRJ(FormatFloat('######0.000', i * int_rec), 11) +'   '+ SetLengthRJ(FormatFloat('#0.0000', RandG(CumD[i], ArtSTEOutflow)),8));
        until (i * int_rec > t_evaporation[1]) or (i * int_rec > t_end);
        SaveToFile(caseName + '_outflow.dat');
        dumint := i;
        if MultiStep = 1 then
          begin
          Clear;
          Add('Hypothetical tensiometer readings');
          Add('   time    Pressure head');
          for i := 1 to counter do
            Add(SetLengthRJ(FormatFloat('######0.000', i * int_rec), 11)+'   '+
                SetLengthRJ(FormatFloat('#######0.0', Ptime[1, i]), 9));
          SaveToFile(caseName + '_pressure.dat');
          end
        else
          begin
        //----------------------------------------------------------
          { Hypothetical sampling for numerical experiment on Two-Stage Method assuming dz = 0.05cm }
          Clear;
          Add('Hypothetical VWC profile');
          Add('z(cm)   VWC');
          for i := 1 to 4 do
            begin
            dum := 0;
            for j := 1 to 10 do dum := dum + W[j+(i-1)*10] / 10;
            Add(FormatFloat('#0.00', i * 0.5) + '   ' + FormatFloat('0.0000', RandG(dum, ArtCEVWC * dum)));
            end;
          for i := 1 to 2 do
            begin
            dum := 0;
            for j := 1 to 20 do dum := dum + W[j+(i-1)*20+40] / 20;
            Add(FormatFloat('#0.00', 2 + i) + '   ' + FormatFloat('0.0000', RandG(dum, ArtCEVWC * dum)));
            end;
          dum := 0;
          for j := 1 to 24 do dum := dum + W[j+80] / 24;
          Add(FormatFloat('#0.00', 5.2) + '   ' + FormatFloat('0.0000', RandG(dum, ArtCEVWC * dum)));
          SaveToFile(caseName+'_profile.dat');
          //----------------------------------------------------------
          Clear;
          Add('Hypothetical water retention data');
          Add('VWC  Pressure(cm)');
          for i := 1 to 4 do
            begin
            dum := 0;
            for j := 1 to 10 do dum := dum + w[j+(i-1)*10] / 10;
            with Soil do
              dum2 := Pressure_Head(dum, 298.2, Wsat_A[1], zeta_A[1], Wr_A[1], 1, Drying[1]);
            if dum2 < -1000 then
              Add(FormatFloat('0.0000',dum)+ '   ' + FormatFloat('##########', RandG(dum2, ArtCEPsychro * dum2)));
            end;
          SaveToFile(caseName+'_psychro.dat');
          //----------------------------------------------------------
          Clear;
          Add('Hypothetical evaporation data');
          Add('time    CumEvaporation');
          i := dumint;
          repeat
            Add(SetLengthRJ(FormatFloat('######0.000', i * int_rec), 11) +'   '+ SetLengthRJ(FormatFloat('#0.0000', RandG(CumE[i], ArtSTEEvap)),8));
            i := i + 6;
          until i > counter;
          Add(SetLengthRJ(FormatFloat('######0.000', counter * int_rec), 11) +'   '+ SetLengthRJ(FormatFloat('#0.0000', RandG(CumE[counter], ArtSTEEvap)),8));
          SaveToFile(caseName+'_evaporation.dat');
          Clear;
          end;
        end;
      end;
  finally
    SL1.Free;
    SL2.Free;
  end;
  if FileExists(Main.GetAbsoluteDirectory(BaseDir, 'BorderDepths.txt')) then
    ShowResult.SliceRings;
end;

procedure TSimulator.Observate;
var
  i, j: Integer;
  zu, zl: Double;
  SL: TStringList;
begin
  time[counter] := time_;
   // W2cmtime(counter) := Walbedo
   // Tair[counter] := Tair;
   // Crystal[counter] := Crystal
  Wb[0] := Wb[1];
  for j := 1 to NumObs do
  begin
    Wtime[j, counter] := (W[NCtime[j]] * (z_obs[j] - z_c[NCtime[j]-1]) + W[NCtime[j]-1] * (z_c[NCtime[j]] - z_obs[j])) / ((dz[NCtime[j]] + dz[NCtime[j]-1]) / 2);
    if (P[NCtime[j]] < 0) and (P[NCtime[j]-1] < 0) then
      Ptime[j, counter] := - exp((ln(- P[NCtime[j]]) * (z_obs[j] - z_c[NCtime[j]-1]) + ln(- P[NCtime[j]-1]) * (z_c[NCtime[j]] - z_obs[j])) / ((dz[NCtime[j]] + dz[NCtime[j]-1]) / 2))
    else
      Ptime[j, counter] := (P[NCtime[j]] * (z_obs[j] - z_c[NCtime[j]-1]) + P[NCtime[j]-1] * (z_c[NCtime[j]] - z_obs[j])) / ((dz[NCtime[j]] + dz[NCtime[j]-1]) / 2);
    Ctime[j, counter] := C[NCtime[j]];
    Ttime[j, counter] := - 273.2 + (Tb[NCtime[j]] * (z_obs[j] - z[NCtime[j]-1]) + Tb[NCtime[j]-1] * (z[NCtime[j]] - z_obs[j])) / dz[nctime[j]];
    AvgWaterContentAbove[j, counter] := GetAvgWaterContentAbove(z_obs[j]);
  end;
  Evap[counter]  := - qv[0]*10*86400/ t_unit;
  if NumExp = 1 then
    if Evap[counter] > MaxEvap then
      MaxEvap := Evap[counter]
    else
      if Evap[counter] < MaxEvap * 0.1 then
        t_end := time_ * 0.99;
  ql_bot[counter] :=  - ql[n]*10*86400/ t_unit;
  CumE[counter] := CumEvap; //qv[0] * 3600 * 24
  CumD[counter] := CumDrain;//ql[n] * 10 * 86400
  CumI[counter] := CumInfilt;//ql[n] * 10 * 86400
  for i := 1 to n do
    if ((W[i] < Wfront) and (W[i+1] > Wfront)) or ((W[i] > Wfront) and (W[i+1] < Wfront)) then
    begin
      zu := 0.5 * (z[i-1] + z[i]);
      zl := 0.5 * (z[i]   + z[i+1]);
      Front[counter] := (zu * (W[i+1] - Wfront) + zl * (Wfront - W[i])) / (W[i+1] - W[i]);
      break;
    end;
  SL := TStringList.Create;
  try
    if rec = 1 then
    begin
      with SL do
      begin
        Clear;
        Add(FormatFloat('##000000.0000', time_) + '  ' + FormatFloat('##0.000', avdt) + '  ' + IntToStr(navdt) + '  ' +
            FormatFloat('0.000000', WCumRError) + '   '+ FormatFloat('0.000000', SCumRError)  + '   '+ FormatFloat('0.000000', ECumRError) + '   '+ FormatFloat('#0.000', Hpond));
        Add('   z(cm)    VWC    psi(cm)    T(C)  c(mg/cm3) VWC*c(mg/cm3) ql(mm/d) qv(mm/d)');
      //  ql[0] := ql[1];
        dum := qv[0];
        qv[0] := qv[1];
        for i := 1 to n do
        begin
          if (z[i] + z[i-1]) / 2 < 10 then dumstr := ' ' else dumstr := '';
          Add('  '+ dumstr + FormatFloat('#0.00',(z[i] + z[i-1]) / 2)+'   '+ FormatFloat('0.0000',W[i]) + '   ' +
               SetLengthRJ(FormatFloat('#######0.0',P[i]),8) + '    ' +
               FormatFloat('#0.0',T[i]-273.2)+'     '+ SetLengthRJ(FormatFloat('##0.00',C[i]),5)+'     '+
               SetLengthRJ(FormatFloat('##0.00',W[i]*C[i]),6) + '       '+ SetLengthRJ(FormatFloat('##0.00', 0.5*(ql[i] + ql[i-1])/t_unit*864000),6) + '    '+ SetLengthRJ(FormatFloat('##0.0', 0.5*(qv[i] + qv[i-1])/t_unit*864000),4)) ;
        end;
        dumstr := BaseDir + Slash + 'History' + Slash;
        if DirectoryExists(dumstr) = False then Createdir(dumstr);
        SaveToFile(dumstr + ExtractFileName(caseName) + '_' + FormatFloat('000', counter) + '.out');
        qv[0] := dum;
      end;
    end;
  finally
    SL.Free;
  end;
end;

function TSimulator.GetAvgWaterContentAbove(Depth: Double): Double;
var i, j: Integer;
begin
  j := 0;
  while z[j] <= Depth do
    j := j + 1;
  if z[1] <> Depth then
  begin
    dum := 0;
    for i := 1 to j - 1 do
      dum := W[i] + dum;
    dum := dum / (j - 1);
  end
  else
    dum := w[1];
  Result := dum;
end;

(*>>>>>>>>>>>>>>>>>>>>>>>   Atmospheric Boundary Condition  <<<<<<<<<<<<<<<<<<<<<<<<<*)
function TSimulator.CalculateAtmos_BC: Boolean;
var
  i : Integer;
  t_prev, t_next: Double;
begin
  i := 0; t_prev := 0; t_next := 0;
  if (abc_type = 3) or (abc_type = 4) then
  begin
    i := abc_done;
    while t_clm_con[i] <= time_ do i := i + 1;
    if i > abc_done then
    begin
      PR[i-1] := ql[0];
      Rg[i-1] := Rshort;
      abc_done := i;
    end;
    t_prev := t_clm_con[i-1];
    t_next := t_clm_con[i];
  end;
  case abc_type of
  1:qv[0] := 0;
  2:begin
      if (time_ < t_evaporation[1]) or (MultiStep = 1) then
        Cv[1] := Cva;
      qv[0] := -(Cv[1] - Cva) / ((ra + rsc)/ t_unit);
    end;
  3:begin
      qv[0] := - (CumEvaporation[i] - CumEvaporation[i-1]) / ( t_next - t_prev );
      if P[1] < -10000 then
      begin
        if P[1] < -1000000 then
        begin
          Result := False;
          Exit;
        end
        else
        begin
          dum := log10(-P[1]);
          qv[0] := (ql[1] * (dum - 4) + qv[0] * (5 - dum));// dum * 10000 / (- P[n]);  //0.5 * ql[5];
        end;
      end;
    end;
  4:begin
    Vwind := (wv[i-1] * ( t_next - time_ ) + wv[i] * (time_ - t_prev )) / ( t_next - t_prev );
    Tair  := (Ta[i-1] * ( t_next - time_ ) + Ta[i] * (time_ - t_prev )) / ( t_next - t_prev );
    RHair := (RHa[i-1]* ( t_next - time_ ) + RHa[i]* (time_ - t_prev )) / ( t_next - t_prev );
    Cva   := RHair * exp(6 - 4976 / Tair);
    if Vwind = 0 then Vwind := 0.1;
    //-- aerodynamic resistance --//
    ra := 0.01 * ln(2000) * ln(2000) / (0.16 * Vwind); //surface roughness=0.1cm
    //ra := ra * (1 + 0.2 * LAI)
    if Tair < T[1] then
    begin
      dum := 9.81 * (2 - 0.01) * (Tair - T[1]) / (Tair * Vwind * Vwind);
      ra := ra / (1 - 10 * dum);
    end;
    qv[0] := -(Cv[1] - Cva) / ((ra + rsc)/ t_unit);
    //------ precipitaion ------//
    dum := CPR[i] - CPR[i-1];
    if dum < 0 then
      dum := 0;
    ql[0] := dum / (t_next - t_prev);
    //--- downward radiation ---//
    dum := CRg[i] - CRg[i-1];
    if dum > 0 then
      Rshort := (dum - Rg[i-1]* (t_next - t_prev))
                 / (( t_next * t_next - t_prev * t_prev ) / 2 - t_prev * ( t_next - t_prev ))
                 * (time_ - t_prev) + Rg[i-1]
    else
      Rshort := 0;
    Rldown := t_unit * (0.605 + 0.048 * Sqrt(1370000 * Cva)) * StephBolz * IntPower(Tair,4) ;
   //------- albedo -------//
    with Soil do
    begin
      if z[1] <> Soil.ThickAlb[1] then
      begin
        Walb := 0;
        for i := 1 to NCalb - 1 do
          Walb := W[i] * dz[i] + Walb;
        Walb_bot := (Wb[NCalb] * (ThickAlb[1] - z[NCalb-1]) + Wb[NCalb-1] * (z[NCalb] - ThickAlb[1])) / dz[NCalb];
        Walb := (Wb[NCalb-1] + Walb_bot) * dz[NCalb] / 2 + Walb;
        Walb := Walb / ThickAlb[1];
      end
      else
        Walb := w[1];
      albedo := AlbedoMin[1] + (AlbedoMax[1] - AlbedoMin[1]) / (1 + Power(Aal[1] * Walb, Bal[1]));
    end;
   //----------------------//
    Rlup := t_unit * (0.9 + 0.18 * w[1]) * StephBolz * IntPower(T[1],4);
    Rl := Rldown - Rlup;
   // Rg := Rg0 * exp(-.82 * LAI)
   // Rl := Rldown * (.5 + .5 * exp(-LAI))
    Rn := (1 - albedo) * Rshort + Rl;
   //-------------------------------------------------------------------------------------------------------//
    qh[0] := Rn + (Tair - T[1]) * 0.0012 / ( 0.5 * ra / t_unit) + ql[0] * 4.18 * (Tair - 1) + qv[0] * 4.18 * T[1];
   //-------------------------------------------------------------------------------------------------------//
   // Please check!! Is ra for heat the same as that of vapor ?
    end;
  end;
  if (abc_type = 2) or (abc_type = 4) then
  begin
    //-- salty crust resistance --//
    if (S_crust = 1) and (Crystal > 0.001) then
    begin
      dum := 0; i := 0;
      repeat
        i := i + 1;
        dum := dum + M[i] * dz[i];
      until z[i] > 0.25;
     // dum := dum + Crystal;
      rsc := 0.39 * ln(dum) - 0.67;
      if rsc < 0.0 then
        rsc := 0.0;
    end
    else
      rsc := 0;
  end;
  Result := True;
end;
(*>>>>>>>>>>>>>>>>>>>>>>>   Water Vapor Movement  <<<<<<<<<<<<<<<<<<<<<<<<<*)
procedure TSimulator.CalculateVaporMovement(ToEnd: Integer);
var
  i, j: Integer;
  chi, Po, Pw, Dvb, Cvb, RHb, ENb: Double;
begin
  //-------------------------------------//
  for i := 1 to ToEnd do
  begin
    if C[i] > 0 then
      chi  := a_oc * C[i] + b_oc * Power(C[i], c_oc) + 1
    else
      chi := 1;
    Po   := -10.2 * (NumIon * C[i] / Mweight) * chi * 8.314 * T[i];
    Pw   := P[i] + Po ;
    RH[i] :=  exp(Pw / 4697 / T[i]);
    Cvsat[i] :=  exp(6 - 4976 / T[i]);
    Cv[i]   := Cvsat[i] * RH[i];
  end;
  with Soil do
    for i := 1 to ToEnd - 1 do
    begin
      j := L[i];
      if heat = 1 then
        En[i] := Aen[j] + Ben[j] * (Wb[i] / Wsat[j]) - (Aen[j] - 1) * exp(-Cen[j] * Power(Wb[i] / Wsat[j], Een[j]))
      else
        En[i] := 1;
      Dv[i] := t_unit * 0.229 * Power(T[i] / 273.2, 1.75) * 0.66* (Wsat[j] - Wb[i+1]); //^ (5 / 3)
      if (Wsat[j] > 0.85) and (z[i] < 0.2) then  // if the porous media is gause,
        Dv[i] := Dv[i] * 2;
      if (Mode = 2) and (IPET_type = 3) and (time_ > t_slicing) and (Abs(z[i] - z_slice) < 0.05) then
        Dv[i] := 0;
    end;
  for i := 1 to ToEnd - 1 do
  begin
    Dvb := (Dv[i] * dz[i+1] / 2 + Dv[i+1] * dz[i] / 2) / ((dz[i] + dz[i+1]) / 2);
    Cvb := (Cv[i] * dz[i+1] / 2 + Cv[i+1] * dz[i] / 2) / ((dz[i] + dz[i+1]) / 2);
    RHb := (RH[i] * dz[i+1] / 2 + RH[i+1] * dz[i] / 2) / ((dz[i] + dz[i+1]) / 2);
    ENb := (En[i] * dz[i+1] / 2 + En[i+1] * dz[i] / 2) / ((dz[i] + dz[i+1]) / 2);
    qv[i] := - Dvb * (ENb * RHb * (Cvsat[i+1] - Cvsat[i]) / ((z[i+1] - z[i-1]) / 2)
             + Cvb / (4712 * Tb[i]) * (P[i+1] - P[i]) / ((z[i+1] - z[i-1]) / 2));
  end;
 //---------------------------------------------//
  for i := 1 to ToEnd do
    Sw[i] := -(qv[i] - qv[i-1]) / dz[i];
end;
(*>>>>>>>>>>>>>>>>>>>>>>>   Liquid Water Flow   <<<<<<<<<<<<<<<<<<<<<<<<<*)
function TSimulator.CalculateLiquidFlow: Boolean;
var
  i, j, j2: Integer;
  Flag, SinkType: Shortint;
  Kmid: TValues;
  t_prev, t_next, Athd, Bthd, Cthd, dum2, P_: Double;
  //Teffect: Double;
begin
  Result   := True;
  Canceled := False;
try
 //---- upper boundary ----//
  SinkType := 1; // initialization
  if abc_type <> 4 then
    case u_bc_type of
    1:ql[0] := ql_top;
    2:if P[1] > Hpond then
        ql[0] := ql[1] * 0.95
      else
        ql[0] := ql[1] * 1.05;
    3:begin
        i := qltop_done;
        while t_ql_top[i] < time_ do
          i := i + 1;
        qltop_done := i;
        t_prev := t_ql_top[i-1];
        t_next := t_ql_top[i];
        if t_next > t_prev then
          ql[0] := (qltop[i-1] * ( t_next - time_ ) + qltop[i] * ( time_ - t_prev )) / ( t_next - t_prev );
      end;
    end;
  if P[1] > 0 then
  begin
   // ql[0] := ql[1] * 0.999;  // fine tuning is required !
    if (u_bc_type <> 2) and ((Hpond > 0) or (ql[0] > 0)) then
      Hpond := (ql[0] - ql[1]) * dt + Hpond;
  end;
  {
  if Hpond > 0 then
  begin
    Pold[1] := Hpond;
    W[1] := Wsat[1] + Soil.Cwsat * Hpond;
  end;
  }
  Sw[1] := Sw[1] + 0.5 * ql[0] / dz[1];
  Sw[2] := Sw[2] + 0.5 * ql[0] / dz[2];
 //---- lower boundary ----//
  case l_bc_type of
  1:  P[n+1] := P_bot + dz[n] / 2;
  2:begin
      P[n+1] := P[n];
      SinkType := 1;
    end;
  4:begin
      i := Pbot_done;
      while t_p_bot[i] < time_ do
        i := i + 1;
      Pbot_done := i;
      t_prev := t_p_bot[i-1];
      t_next := t_p_bot[i];
      if t_next > t_prev then
        P[n+1] := (Pbot[i-1] * ( t_next - time_ ) + Pbot[i]* ( time_ - t_prev )) / ( t_next - t_prev );
    end;
  5:begin
      if (t_outflow[1] = 0) and (OutFlow_done = 1) then
        OutFlow_done := 2;
      i := OutFlow_done;
      while (t_outflow[i] < time_) and (i < NumOutflowData_for_DP) do
        i := i + 1;
      OutFlow_done := i;
      if SinkType = 1 then
      begin
        //dum := (CumOutFlow[i-1] * ( t_next - (time_ + dt)) + CumOutFlow[i]* ((time_ + dt) - t_prev )) / ( t_next -  t_prev );
        dum := (CumOutFlow[i] - CumOutFlow[i-1]) / ( t_outflow[i] - t_outflow[i-1] );
        if P[n] > - 10000 then
          ql[n] := dum
        //  ql[n] := (dum - CumDrain) / dt
        else
        begin
          {
          if P[n] > - 10000 then
            ql[n] := dum * 5000 / - P[n]
          else
          }
          begin
            Result := False;
            Application.ProcessMessages;
            Exit;
          end;
        end;
        Sw[n] := Sw[n] - ql[n] / dz[n];
      end;
      P[n+1] := P[n];
      //--------------
      {
      i := Pbot_done;
      while t_p_bot[i] < time_ do
        i := i + 1;
      Pbot_done := i;
      t_prev := t_p_bot[i-1];
      t_next := t_p_bot[i];
      if t_next > t_prev then
        dum := (Pbot[i-1] * ( t_next - time_ ) + Pbot[i]* ( time_ - t_prev )) / ( t_next - t_prev );
      if SinkType = 0 then
        P[n+1] := dum;
      }
    end;
  end;
  with Soil do
    W[n+1] := Water_Content(P[n+1], T[n+1], Wsat_A[n+1], zeta_A[n+1], Wr_A[n+1], NumLayer, Drying[n+1]);
  for i := 1 to n do
  begin
    P[i]     := Pold[i];
    Wtent[i] := W[i];
  end;
  Wtent[n+1] := W[n+1];
  case IterationScheme of
  0: iteration := 20;
  1: iteration := 19;
  2: iteration :=  0;
  end;
  //=================================================
  repeat
    iteration := iteration + 1;
    NumCalc := NumCalc + 1;
    //---- Unsaturated Hydraulic Conductivity ----//
    if InterHyCon < 2 then
      for i := 1 to n + 1 do
      begin
        //if Wtent[i] > Soil.Wsat[1] then
        //  ShowMessage(FloatToStr(Wtent[i]));
        if C[i] > 1 then
          dum := 1 / (1 + 0.0029 * Power(0.1 * C[i], 1.77))
        else
          dum := 1;
        Kmid[i] := dum * Soil.HydraulicConductivity(0.5 * (W[i] + Wtent[i]), T[i], L[i]);
      end;
    for i := 1 to n do
    begin
      //---- Fingering ----//
      {
      if ((W[i] < 0.01) and (P[i+1] < -5)) or ((W[i+1] < 0.01) and (P[i] < -5)) then
        K[i] := 0
      else
      }
      begin
      //  if (iteration = 1) or ((W[i] > 0.01) and (W[i+1] > 0.01)) then
        begin
      //    if (time_ > 0) and (K[i] = 0) and (dt > 0.0001) then
      //      dt := 0.00000001 * dt;
          case InterHyCon of
          0:begin
              if Kmid[i] < Kmid[i+1] then
                K[i] := 0.75 * Kmid[i] + 0.25 * Kmid[i+1]
              else
                K[i] := 0.25 * Kmid[i] + 0.75 * Kmid[i+1];
              //K[i] := 0.5 * (Kmid[i] + Kmid[i+1]);
            end;
          1: K[i] := Sqrt(Kmid[i] * Kmid[i+1]);
          2: K[i] := Soil.HydraulicConductivity(0.25 * (W[i] + W[i+1] + Wtent[i] + Wtent[i+1]), T[i], L[i]);
          end;
          with Soil do
            if K[i] > t_unit * Ksat[L[i]] then
              K[i] := t_unit * Ksat[L[i]];  // for fingering
        end;
      end;
    end;
    for i := 1 to n + 1 do
    begin
      if C[i] > 20 then
      begin
        rho[i] := 0.001 * (1 - 0.4) * C[i] + 1;  // density of water
        K[i] := K[i] / (15.2 * Sqr(0.001 * 0.5 * (C[i] + C[i+1])) + 0.318 * 0.001 * 0.5 * (C[i] + C[i+1]) + 1); // viscosity effect
      end
      else
        rho[i] := 1;
      if (Mode = 2) and (IPET_type = 3) and (time_ > t_slicing) and (Abs(z[i] - z_slice) < 0.05) then
        K[i] := 0;
    end;
    //--- lower boundary ---//
    if l_bc_type = 3 then
      K[n] := 0;
    if (l_bc_type = 5) and (SinkType = 1) then
      K[n] := 0;
    if (l_bc_type = 1) or (l_bc_type = 4) or (SinkType = 0) then
      if (HC_filter > 0) and ((IPET_type <> 2) or (UseOutFlowForK = 1) or (l_bc_type <> 4)) then // and (Tensio <> 2)
        K[n] := HC_filter * t_unit * dz[n];
      //  K[n] := 1 / (1 / HC_filter + Abs(parlb_a) * Power(CumDrain + 0.00001, Parlb_b)) * t_unit * dz[n];
    if (NumExp = 1) or (IPET_type = 1) then
      if (time_ >= t_evaporation[1]) and (MultiStep <> 1) then
        K[n] := 0;
  //----- Water Capacity -----//
    for i := 1 to n do
    begin
      j := L[i];
      if IterationScheme = 1 then
        P_ := 0.5 * (P[i] + Pold[i])
      else
        P_ := P[i];
      with Soil do
        Cw[i] := Water_Capacity(P_, T[i], Wsat_A[i], zeta_A[i], Wr_A[i], j, Drying[i]);
    end;
    //------- Seepage face -------//
    if l_bc_type = 6 then
    begin
      if P[n] > - dz[n] / 2 then
        P[n+1] := dz[n] / 2   // free drainage
      else
        P[n+1] := P[n] + dz[n+1]; // zero flux
    end;
    with Soil do
      W[n+1] := Water_Content(P[n+1], T[n+1], Wsat_A[n+1], zeta_A[n+1], Wr_A[n+1], NumLayer, Drying[n+1]);
    //------- Building of Coefficient Matrix -------//
    if IterationScheme = 1 then
    begin
      for i := 1 to n do
        P[i] := Pold[i];
    end;
  //  CalculateVaporMovement(8);
    if (iteration = 1) or (IterationScheme < 2) then
      Flag := 0
    else
      Flag := 1;
    for i := 1 to n do
    begin
      U[i]    := Gravity * (rho[i] * K[i] - rho[i-1] * K[i-1]) / dz[i] - Sw[i] + Flag * (Wtent[i] - W[i]) / dt;
      Ath[i+1]:=   2 * ETA *  K[i] / (dz[i+1] * (z[i+1] - z[i-1]));
      Bth[i]  := -2 * ETA * (K[i] / (dz[i]   * (z[i+1] - z[i-1])) + K[i-1] / (dz[i] * (z[i] - z[i-2]))) - Cw[i] / dt;
      Cth[i]  :=  2 * ETA *  K[i] / (dz[i]   * (z[i+1] - z[i-1]));
      Athd    := -2 * (1 - ETA) *  K[i-1] / (dz[i] * (z[i]   - z[i-2])) * P[i-1];
      Bthd    := (2 * (1 - ETA) * (K[i]   / (dz[i] * (z[i+1] - z[i-1]))
                    + K[i-1] / (dz[i] * (z[i] - z[i-2]))) - Cw[i] / dt) * P[i];
      Cthd    := -2 * (1 - ETA) *  K[i]   / (dz[i] * (z[i+1] - z[i-1])) * P[i+1];
      Dth[i]  := Athd + Bthd + Cthd + U[i];
    end;
    Dth[n] := Dth[n] - Cth[n] * P[n+1]; // 01/06/01 changed
    SolveTheCoefficientMatrix;
    MaxDiffW := 0;
    for i := 1 to n do
    begin
     // if x[i] < 100 then
      if (x[i] < 100) and (x[i] > -10000000000) then
        P[i] := x[i]
      else
      begin
        dt := dt;
        {
        dt := dt / 10;
        CanceledTime := CanceledTime + dt;
        Canceled := True;
        Exit;
        }
      end;
      //if P[i] > 200 then
      //  P[i] := 0;  // For fingering
      j := L[i];
      with Soil do
        dum := Water_Content(P[i], T[i], Wsat_A[i], zeta_A[i], Wr_A[i], j, Drying[i]);
      if MaxDiffW < ABS(dum - Wtent[i]) then
        MaxDiffW := ABS(dum - Wtent[i]);
      Wtent[i] := dum;
    end;
  until ((MaxDiffW < CriteriaForCeliasIteration) or (iteration > 20));
  //---- Water Budget Check -----//
  W_storage_new := 0;
  for i := 1 to n do
    W_storage_new := Wtent[i] * dz[i] + W_storage_new;
  ql[n] := -K[n] * ((Pold[n+1] - (ETA * P[n] + (1 - ETA) * Pold[n])) / ((z[n+1] - z[n-1])/2) - 1);
  if (l_bc_type = 5) and (SinkType = 1) then
    ql[n] := - Sw[n] * dz[n];
  // Each Step Relative Error for Water
  ESRErrorW := ABS((W_storage_new - W_storage_old) - (ql[0] + qv[0] - ql[n]) * dt) / W_storage_new;
  if (ql[0] + qv[0] - ql[n]) <> 0 then
    ESErrorW := ((W_storage_new - W_storage_old) - (ql[0] + qv[0] - ql[n]) * dt) / ABS((ql[0] + qv[0] - ql[n]) * dt);
  MaxDiff_logP := 0;
  MaxDiffW := 0;
  for i := 1 to n do
  begin
    if (Pold[i] <> 0) then
      MaxDiff_logP := Max(MaxDiff_logP, ln(Abs(Pold[i])/Abs(P[i])));
    Pold[i] :=  P[i];
    W[i]   := Wtent[i];
    j := L[i];
    //----- Handling of Hysteresis ----//
    if (ConsiderHysteresis = 1) and (Mode <> 2) then
      if Drying[i] then
      begin
        if ((W[i] - Wold[i]) / dt * t_conv > Margin) then
        begin
          SetScanningCurve(j, i, False);
          Drying[i] := False;
        end;
      end
      else
      begin
        if ((W[i] - Wold[i]) / dt * t_conv < -Margin) then
        begin
          SetScanningCurve(j, i, True);
          Drying[i] := True;
        end;
      end;
    //if W[i] > Wsat[j] then
    //  W[i] := Wsat[j] - 0.0001;
    Wold[i]   := W[i];
  end;
  Pold[n+1] :=  P[n+1];
  //--- Flux for Graphical Output and Solute Transfer---//
  for i := 1 to n do
    ql[i] := -K[i] * ((P[i+1] - P[i]) / ((z[i+1] - z[i-1]) / 2) - Gravity * rho[i]);
  if (l_bc_type = 5) and (SinkType = 1) then
    ql[n] := - Sw[n] * dz[n];
  //---- Cumulative Values for Water Budget ----//
  CumEvap   := -qv[0] * dt + CumEvap;
  CumInfilt :=  ql[0] * dt + CumInfilt;
  CumDrain  := ql[n] * dt + CumDrain;
  if Abs(CumInfilt - CumDrain - CumEvap) >  W_storage_ini then
    WCumRError :=  ABS((W_storage_new - W_storage_ini) - (CumInfilt - CumDrain - CumEvap)) / Abs(CumInfilt - CumDrain - CumEvap)
  else
    WCumRError :=  ABS((W_storage_new - W_storage_ini) - (CumInfilt - CumDrain - CumEvap)) / W_storage_ini;
  W_storage_old := W_storage_new;
  //-- moving pressure lower boundary for fingering study --//
  if MLB then
  begin
    ql[n] := 0;
    if CumDrain > Soil.Wsat[1] * dz[n] then
    begin
      j := Trunc(CumDrain / (Soil.Wsat[1] * dz[n]));
      CumDrain := CumDrain - j * (Soil.Wsat[1] * dz[n]);
      for i := 1 to j do
      begin
        P[n+i+1]    := P[n+1];
        Pold[n+i+1] := P[n+1];
        W[n+i+1]    := W[n];
        T[n+i+1]    := T[n];
        Tb[n+i]     := T[n];
        C[n+i+1]    := C[n+1];
        L[n+i+1]    := L[n+1];
        j2 := L[n+i+1];
        Wsat_A[n+i+1] := Soil.Wsat[j2];
        zeta_A[n+i+1] := Soil.zeta[j2];
        Wr_A[n+i+1]   := Soil.Wr[j2];
        Drying[n+i+1] := Drying[n+i];
        dz[n+i+1]   := dz[n+1];
        z[n+i+1]    := z[n+i] + dz[n+1];
        z_lb := z[n];
      end;
      n := n + j;
      z_lb := z[n];
    end;
  end;
  //----  at the compartment boundaries ----//
  W[n+1] := W[n];  W[0] := W[1]; // imaginary values.
  for i := 1 to n do
    Wb[i] := (W[i] * dz[i+1] / 2 + W[i+1] * dz[i] / 2) / ((dz[i] + dz[i+1]) / 2);
  Wb[n+1] := Wb[n];
  //---- calculation of mean pore water velocity for CRWM ---//
  if CRWM = 1 then
  begin
    Vmax := 0;
    for i := 1 to n do
    begin
      V[i] := ql[i] / Wb[i];
      if Abs(V[i]) > Vmax then
        Vmax := Abs(V[i]);
    end;
    dt_CRWM_next := 2 * Soil.dispersivity[1] / Abs(Vmax);
    if Vmax * (86400 / t_end) > 1000 then
      dt_CRWM_next := dt_CRWM_next / 20;
    if time_CRWM + dt_CRWM_next > int_rec * (counter + 1) then
      dt_CRWM_next := int_rec * (counter + 1) - time_ - 0.001;
  end;
except
  ShowMessage(FloatToStr(time_));
  Application.ProcessMessages;
  Application.Terminate;
end;
end;

(*>>>>>>>>>>>>>>>>>>>>>>>   Solute  Movement  <<<<<<<<<<<<<<<<<<<<<<<<<*)

procedure TSimulator.CalculateSoluteMovement;
var
  i, i2, i3, j: Integer;
  Athd, Bthd, Cthd, t_prev, t_next, dispersivity_: Double;
begin
  if CRWM = 1 then
  begin
    dz_ := (z[n+1] - z[n-1]) / 2;
    if water <> 1 then
    begin
      ql[0] := 0.3 * Vfixed;
      ql[n] := 0.3 * Vfixed;
    end;
    if ql[n] > 0 then
      qs[n] := -Wb[n] * Dm[n] * (C[n+1] - C[n]) / dz_ + ql[n] * C[n]
    else
      qs[n] := ql[n] * C[n+1]; // -Wb[n] * Dm[n] * (C[n+1] - C[n]) / dz_ ;
    if ql[0] > 0 then
      qs[0] := ql[0] * C_irg
    else
      qs[0] := 0;
    if dumint <> 12345 then
    begin
      Cum_qs_bot := Cum_qs_bot + qs[n]* dt;
      Cum_qs_top := Cum_qs_top + qs[0]* dt;
      dumint := 0;
    end
    else
      dumint := 0;
    dt_CRWM := time_ - time_CRWM;
    if dt_CRWM < dt_CRWM_next then
      Exit
    else
      time_CRWM := time_;
  end;
 //---- upper boundary ----//
  if s_u_bc_type = 2 then
  begin
    i := Cirg_done;
    while t_c_irg[i] < time_ do
      i := i + 1;
    Cirg_done := i;
    t_prev := t_c_irg[i-1];
    t_next := t_c_irg[i];
    if t_next > t_prev then
      C_irg := (Cirg[i-1] * ( t_next - time_ ) + Cirg[i]* ( time_ - t_prev )) / ( t_next - t_prev );
  end;
  qs[0] := ql[0] * C_irg;
 //---- lower boundary ----//
  if s_l_bc_type = 2 then
  begin
    i := Cbot_done;
    while t_c_bot[i] < time_ do
      i := i + 1;
    Cbot_done := i;
    t_prev := t_c_bot[i-1];
    t_next := t_c_bot[i];
    if t_next > t_prev then
      C_bot := (Cbot[i-1] * ( t_next - time_ ) + Cbot[i] * ( time_ - t_prev )) / ( t_next - t_prev );
  end;
  case s_l_bc_type of
  1: C[n+1] :=  C_bot;
  2: C[n+1] :=  C_bot;
  3: C[n+1] :=  C[n];
  end;
 //------------------------
  Wb[0] := w[1];
  Wb[n+1] := W[n];
  Tb[0] := Tb[1] ;
  ql[n+1] :=  ql[n];
  NegativeDCwarning := 0;
  Vmax := 0;
  if (water = 1) or ((Mode = 2) and (time_ = 0)) then
    for i := 1 to n do
    begin
      j := L[i];
      if (water = 1) or (Mode = 2) then
      begin
        V[i] := ql[i] / Wb[i];
        with Soil do
        begin
          //-- molecular Diffusion coefficient --//
          Di[i] := Diw0 * (a_tau[j] *  (1 - exp( - Power(b_tau[j] * Wb[i], c_tau[j]))) + d_tau[j] * Wb[i]);
          //-- mechanical Dispersion coefficient--//
          if (CDE_surface_modification) and (z[i] < 2) then
            dispersivity_ := dispersivity[j] * (0.333 * z[i] + 0.333)
          else
            dispersivity_ := dispersivity[j];
          Dm[i] := dispersivity_ * ABS(V[i]);
          //--------------------------------------//
        end;
      end
      else
      begin
        Dm[i] := Ds;
        V[i]  := Vfixed;
      end;
      if Abs(V[i]) > Vmax then
        Vmax := Abs(V[i]);
      //-- collected Dispersion coefficient --//
      dz_ := (z[i+1] - z[i-1]) / 2;
      Dc[i] := Dm[i] + Di[i] - (ABS(V[i]) * dz_ / 2) - V[i] * V[i] * dt / 8 / Wb[i] * (Wb[i] - Wbold[i]);
      if (Dc[i] < 0) and (CRWM <> 1) then
      begin
        Dc[i]  := 0.000001;
       // NegativeDCwarning := 1;
       // time_ := t_end + 1;
      end;
      Wbold[i] := Wb[i];
      //-- solute flux --//
     // if ql[i] > 0 then
     //   qs[i] := -Wb[i] * Dc[i] * (C[i+1] - C[i]) / dz_ + ql[i] * C[i]
     // else
     //   qs[i] := -Wb[i] * Dc[i] * (C[i+1] - C[i]) / dz_ + ql[i] * C[i+1];
    end;
  if CRWM <> 1 then
  //=====  Convection Dispersion Equation =====//
  begin
    //-- Implicit FDM --//
    for i := 1 to n do
    begin
      //-- non-equilibrium crystallization --//
      if (alpha_crys < 0.01) and (Crystl[i] > 0) and (water = 1) then  // and (ql[i] > 0)
      begin
        Ss[i] := alpha_crys * t_unit * W[i] * (Cmax - C[i]);
        Crystl[i] := Crystl[i] - Ss[i] * dt;
      end
      else
        Ss[i] := 0;
      //-------------------------------------//
    end;
    Ss[1] := Ss[1] + qs[0] / dz[1];
    for i := 0 to n do
      if ql[i] > 0 then
        pi[i] := 1
      else
        pi[i] := 0;
    for i := 1 to n do
    begin
      Ath[i+1]:=  ETAs / dz[i+1]* (2 * Wb[i] * Dc[i] / (z[i+1] - z[i-1]) + ql[i] * pi[i]);
      Bth[i]  := -ETAs / dz[i]  * (2 * Wb[i] * Dc[i] / (z[i+1] - z[i-1]) + 2 * Wb[i-1] * Dc[i-1] / (z[i] - z[i-2]) + ql[i] * pi[i] - ql[i-1] *  (1 - pi[i-1])) - W[i] / dt ;
      Cth[i]  :=  ETAs / dz[i]  * (2 * Wb[i] * Dc[i] / (z[i+1] - z[i-1]) - ql[i] * (1 - pi[i]));
      Athd := -(1 - ETAs)/ dz[i]  * (2 * Wb[i-1]* Dc[i-1]/ (z[i]   - z[i-2]) + ql[i-1] * pi[i-1])  * C[i-1];
      Bthd := ((1 - ETAs)/ dz[i]  * (2 * Wb[i]  * Dc[i]  / (z[i+1] - z[i-1]) + 2 * Wb[i-1] * Dc[i-1] / (z[i] - z[i-2]) + ql[i] * pi[i] - ql[i-1] *  (1 - pi[i-1])) - Wolds[i] / dt) * C[i];
      Cthd := -(1 - ETAs)/ dz[i]  * (2 * Wb[i]  * Dc[i]  / (z[i+1] - z[i-1]) - ql[i] * (1 - pi[i]))* C[i+1];
      Dth[i] := Athd + Bthd + Cthd - Ss[i] ;
    end;
    Dth[n] := Dth[n] - Cth[n] * C[n+1];  // 01/06/01 changed

    SolveTheCoefficientMatrix;
    Crystal := 0;
    for i := 1 to n do
    begin
      C[i] := x[i];
      Wolds[i] := W[i];
    //  M[i] := W[i] * C[i];
      //-- equilibrium crystallization --//
      if (alpha_crys > 0.01) and ((C[i] > Cmax) or (Crystl[i] > 0)) and (water = 1) then
      begin
        Crystl[i] := Crystl[i] - W[i] * (Cmax - C[i]);
        if Crystl[i] > 0 then
          C[i] := Cmax
        else
        begin
          C[i] := Cmax + Crystl[i] / W[i];
          Crystl[i] := 0;
        end;
      end;
      Crystal := Crystal + Crystl[i] * dz[i];//(M[i] - W[i] * Cmax) * dz[i];
      M[i] := W[i] * C[i] + Crystl[i];
    end;
    //-- mass conservation --//
    {
    for i := 1 to n do
      begin
      M[i] := -(qs[i] - qs[i-1]) / dz_ * dt + M[i];
      C[i] := M[i] / W[i];
      end;
    }
  end
  else
  begin
    //=======  Convective Random Walk Model  =======//
    //-- determination of a,Ym and Y0  --//
    Dm[n+1] := Dm[n];
    for i := 1 to n do
      begin
      //-- v modification (Uffink,1985) --
      // v[i] := v[i] + (Dm[i+1] - Dm[i]) / dz[i];
      //----------------------------------
      if v[i] < 0 then SIGN[i] := -1 else SIGN[i] := 1;
      mean_[i] := ABS(v[i]) * dt_CRWM;
      //STD := Sqrt(2 * Dm[i] * dt_CRWM);
      Xmax[i] := 6 * Soil.dispersivity[L[i]] + 1.5 * mean_[i];
      Ym[i]   := 4 * mean_[i] / Sqr(Xmax[i]);
      Y0[i]   := (2 * Xmax[i] - 4 * mean_[i]) / (mean_[i] * Xmax[i])
      end;
    //-- moving of particle --//
    for i := 1 to NumP do
      if (zp[i] > 0) and (zp[i] < z_lb) then
        begin
        OneStepDisp := GetOneStepDisp(ncp[i]);
        if water =1 then
          for i2 := 1 to 50 do
            begin
            RN := -2; //=  6 / 3
            for i3 := 1 to 4 do RN := RN + Random; // 4 = 12 / 3
            //'  RN0 := RND[1]
            //'  RN := (RN0 - .5) * 12 ^ .5
            OneStepDiff := RN * Sqrt(6 * Di[ncp[i]] * dt_CRWM / 50); // 6 = 3 * 2
            zp[i] := zp[i] + OneStepDisp / 50 + OneStepDiff;
            if zp[i] <= 0 then zp[i] := 0.01;// else if zp[i] >= z_lb then zp[i] := z_lb - 0.01;
            if (Abs(V[n]* (86400 / t_unit)) < 0.5) and (zp[i] > z_lb) then zp[i] := z_lb - (zp[i] - z_lb);  // reflective lower boundary
            if (ql[n] < 0) and (zp[i] > z_lb) then zp[i] := z_lb - (zp[i] - z_lb);  // reflective lower boundary
            end
        else
          zp[i] := zp[i] + OneStepDisp;
        end;
    //--- making of particle ---//
    // lower boundary
    Integler_lb := - C_bot * ql[n] * dt_CRWM + Integler_lb;
    //if Integler_lb < 0 then Integler_lb := 0;
    if Integler_lb > MPP then
    begin
      mean_[n] := ABS((V[n] + Vnbefore) / 2) * dt_CRWM;
      //STD := Sqrt(2 * Dm[i] * dt_CRWM);
      Xmax[n] := 6 * Soil.dispersivity[NumLayer] + 1.5 * mean_[n];
      Ym[n]   := 4 * mean_[n] / Sqr(Xmax[n]);
      Y0[n]   := (2 * Xmax[n] - 4 * mean_[n]) / (mean_[n] * Xmax[n]);
      Vnbefore := V[n];
      dumint := NumP;
      NumP := NumP + Trunc(-C_bot * ql[n] * dt_CRWM / MPP);
      for i := dumint + 1 to NumP do
      begin
        ncp[i] := n;
        repeat
          zp[i] := z_lb + Xmax[n] * Random;
          OneStepDisp := GetOneStepDisp(n);
        until zp[i] + OneStepDisp < z_lb - 0.000001;
        zp[i] := zp[i] + OneStepDisp;
      end;
      Integler_lb := Integler_lb - MPP * INT(-C_bot * ql[n] * dt_CRWM / MPP);
    end;
    // surface boundary
    Integler_ub := C_irg * ql[0] * dt_CRWM + Integler_ub;
    if Integler_ub > MPP then
      mean_[1] := ABS((V[1] + V1before) / 2) * dt_CRWM;
      //STD := Sqrt(2 * Dm[i] * dt_CRWM);
      Xmax[1] := 6 * Soil.dispersivity[1] + 1.5 * mean_[1];
      Ym[1]   := 4 * mean_[1] / Sqr(Xmax[1]);
      Y0[1]   := (2 * Xmax[1] - 4 * mean_[1]) / (mean_[1] * Xmax[1]);
      V1before := V[1];
    begin
      dumint := NumP;
      NumP := NumP + Trunc(C_irg * ql[0] * dt_CRWM / MPP);
      for i := dumint + 1 to NumP do
      begin
        ncp[i] := 1;
        repeat
          zp[i] := - Xmax[1] * Random;
          OneStepDisp := GetOneStepDisp(1);
        until zp[i] + OneStepDisp > 0.000001;
        zp[i] := zp[i] + OneStepDisp;
      end;
      Integler_ub := Integler_ub - MPP * INT(C_irg * ql[0] * dt_CRWM / MPP);
    end;
    //---------------------------
    for i := 1 to n do
      M[i] := 0;
    for i := 1 to NumP do
      if (zp[i] >= 0) and (zp[i] <= z_lb) then
      begin
        if com_ratio = 1 then
          ncp[i] := Trunc(zp[i]/dz[1]) + 1
        else
          ncp[i] := Ceil(LogN(com_ratio, 1 + zp[i] * (com_ratio - 1) / dz[1]));
        M[ncp[i]] := M[ncp[i]] + MPP / dz[ncp[i]];
      end;
    for i := 1 to n do C[i] := M[i] / W[i];
    //--- crystallization ---//
    i := 0;
    if C[1] > 310 then
      while M[1] / W[1] > 310 do
      begin
        i := i + 1;
        if (ncp[i] = 1) and (zp[i] > 0) then
        begin
          M[1] := M[1] - MPP / dz[1];
          zp[i] :=  -1;
          Crystal := Crystal + MPP;
        end;
      end
    else if Crystal > 0 then
    //--- dissolution ---//
      while M[1] / W[1] < 310 do
      begin
        i := i + 1;
        if i > NumP then break;
        if zp[i] = -1 then
        begin
          M[1] := M[1] + MPP / dz[1];
          zp[i] :=  0.01;
          Crystal := Crystal - MPP;
        end;
      end;
    //---------------------//
    C[1] := M[1] / W[1];
    if water <> 1 then
    begin
      dt_CRWM_next := 2 * Soil.dispersivity[1] / Abs(Vmax);
      if time_ + dt_CRWM_next > int_rec * (counter + 1) then
        dt_CRWM_next := int_rec * (counter + 1) - time_ - 0.001;
    end;
  end;
//===================================================================
  dz_ := (z[n+1] - z[n-1]) / 2;
  if CRWM <> 1 then
    if ql[n] > 0 then
      qs[n] := -Wb[n] * Dc[n] * (C[n+1] - C[n]) / dz_ + ql[n] * C[n]
    else
      qs[n] := -Wb[n] * Dc[n] * (C[n+1] - C[n]) / dz_ + ql[n] * C[n+1];
 //---- Mass Balance ---//
  S_storage := 0;
  for i := 1 to n do
    S_storage := S_storage + M[i] * dz[i];
  if CRWM <> 1 then
  begin
    Cum_qs_bot := Cum_qs_bot + qs[n]* dt;
    Cum_qs_top := Cum_qs_top + qs[0]* dt;
  end;
  if S_storage_ini > S_storage then
    dum := S_storage_ini
  else
    dum := S_storage;
  if dum > 0 then
    SCumRError := ABS((S_storage - S_storage_ini) - (Cum_qs_top - Cum_qs_bot)) / dum
  else
    SCumRError := 0;
end;

(*>>>>>>>>>>>>>>>>>>>>>>>   Heat Movement  <<<<<<<<<<<<<<<<<<<<<<<<<*)

procedure TSimulator.CalculateHeatMovement;
var
  i, j: Integer;
  Athd, Bthd, Cthd: Double;
begin
  with Soil do
    for i := 1 to n + 1 do
    begin
      j := L[i];
      Chnew[i] := (2.13 * (1 - Wsat[j]) + 4.18 * W[i]);
      kh[i] := t_unit * (Akh[j]  + Bkh[j] * (Wb[i] / Wsat[j]) - (Akh[j] - Dkh[j]) * exp(-Ckh[j] * Power(Wb[i] / Wsat[j], Ekh[j])));
    end;
  if h_l_bc_type = 2 then
    kh[n] := 0;
 //-- Lower Boundary Condition --//
  qh[n] := -kh[n] * (T[n+1] - T[n]) / ((z[n+1] - z[n-1]) / 2) + ql[n] * 4.18 * (pi[n] * T[n] + (1 - pi[n]) * T[n+1]);
 //--energy conservation--
  for i := 1 to n do
  begin
    LH[i] := (597 - 0.6 * (T[i] - 273.2)) * 4.18;
    Sh[i] := -LH[i] * (qv[i] - qv[i-1]) / dz[i] - LOSS * (T[i] - Tini);
  end;
 //------- Building of Coefficient Matrix -------//
  Sh[1] := Sh[1] + qh[0] / dz[1];
  for i := 0 to n do
    if ql[i] > 0 then pi[i] := 1 else pi[i] := 0;
  Ch[0] := Ch[1]; Ch[n+1] := Chnew[n+1];//tentative
  if h_l_bc_type = 1 then
    h[n+1] :=  Tbot * Chnew[n+1];
  for i := 1 to n do
  begin
    Ath[i+1] :=  ETAh / (Chnew[i]  * dz[i+1])* (2 * kh[i]  / (z[i+1] - z[i-1]) + ql[i] * 4.18 * pi[i]);
    Bth[i]   := -ETAh / (Chnew[i]  * dz[i])  * (2 * kh[i]  / (z[i+1] - z[i-1]) + 2 * kh[i-1] / (z[i] - z[i-2]) + ql[i] * 4.18 * pi[i] - ql[i-1] * 4.18 * (1 - pi[i-1])) - 1 / dt;
    Cth[i]   :=  ETAh / (Chnew[i+1]* dz[i])  * (2 * kh[i]  / (z[i+1] - z[i-1]) - ql[i] * 4.18 * (1 - pi[i]));
    Athd := -(1 - ETAh) / (Ch[i-1]* dz[i])   * (2 * kh[i-1]/ (z[i]   - z[i-2]) + ql[i-1] * 4.18 * pi[i-1]) * h[i-1];
    Bthd := ((1 - ETAh) / (Ch[i]  * dz[i])   * (2 * kh[i]  / (z[i+1] - z[i-1]) + 2 * kh[i-1] / (z[i] - z[i-2]) + ql[i] * 4.18 * pi[i] - ql[i-1] * 4.18 * (1 - pi[i-1])) - 1 / dt) * h[i];
    Cthd := -(1 - ETAh) / (Ch[i+1]* dz[i])   * (2 * kh[i]  / (z[i+1] - z[i-1]) - ql[i] * 4.18 * (1 - pi[i])) * h[i+1];
    Dth[i] := Athd + Bthd + Cthd - Sh[i];
  end;
  if h_l_bc_type = 1 then
    Dth[n] := Dth[n] - Cth[n] * h[n+1]; // 01/06/01 changed
  heatcalc := 1;
 //-----//
  SolveTheCoefficientMatrix;
 //-----//
  heatcalc := 0;
  for i := 1 to n do
  begin
    h[i] := x[i];
    Tnew := h[i] / Chnew[i];
    Ch[i] :=  Chnew[i];
    if ABS(Tnew - T[i]) > maxchangeT then maxchangeT := ABS(Tnew - T[i]);
    P[i] := P[i] * exp(0.0068 * (T[i] - Tnew));
    Pold[i] := P[i];
    T[i] := Tnew;
  end;
 //-- Boundary Value --//
  for i := 1 to n do
    Tb[i] := (T[i] * dz[i+1] / 2 + T[i+1] * dz[i] / 2) / ((dz[i] + dz[i+1]) / 2);
 //--- Energy Balance ---//
  E_storage := E_storage_old;
  E_storage := 0;
  for i := 1 to n do  E_storage := E_storage + h[i] * dz[i];
  TotalLoss := 0;
  for i := 1 to n do  TotalLoss := TotalLoss + LOSS * (T[i] - Tair) * dz[i];
  //rerrore := ABS((EStoRAGENEW - EStoRAGEOLD) - (qh[0] - qh[n] - toTALHLOSS + qv[0] * LH[1)) * dt) / EStoRAGENEW 'Each Step Relative Error
  //eserrore := ABS((EStoRAGENEW - EStoRAGEOLD) - (qh[0] - qh[n] - toTALHLOSS + qv[0] * LH[1)) * dt) / ((qh[0] - qh[n] - toTALHLOSS + qv[0] * LH[1)) * dt) 'Each Step Error
  CumSHT := qh[0] * dt - (qv[0] * 4.18 * T[1] * dt + ql[0] * 4.18 * (Tair - 1) * dt) + CumSHT;//Cumulative sensible heat transfer
  CumLHT := qv[0] * LH[1] * dt + CumLHT;   // Cumulative latent heat transfer
  CumHLB := qh[n] * dt + CumHLB;           // Cumulative heat loss from the bottom
  CumHLS := TotalLoss * dt + CumHLS;       // Cumulative heat loss from the side
  CumCHL := qv[0] * 4.18 * T[1] * dt + ql[0] * 4.18 * (Tair - 1) * dt + CumCHL; // Cumulative convective heat loss from the surface
  ECumRError := ABS((E_Storage - E_Storage_ini) - (CumSHT + CumLHT + CumCHL - CumHLB - CumHLS)) / E_Storage_ini;
  //if ABS(ECumRError) > .1 then PRINT "Cumulative relative error for heat transfer is too large!,"; eCumrerror
end;
//==============================================================================================
procedure TSimulator.DetermineSpaceIncrements;
var
  c: Double;
  i, j: Integer;
begin
  z_border[NumLayer] := z_lb;
  com_ratio :=  (z_lb - z[1]) / (z_lb - dz_bot);
  z[0] := 0;
  if com_ratio = 1 then
  begin
    n := Trunc(z_lb / z[1] + 0.00001);
    for i := 1 to n do
    begin
      dz[i] := z[1];
      z[i] := z[i-1] + dz[i];
      z_c[i] := (z[i] + z[i-1]) / 2;
    end;
  end
  else
  begin
    n := Trunc(ln(z_lb * (com_ratio - 1) / z[1] + 1) / ln(com_ratio) + 0.00001);
    c := FindRoot(-0.01, 0.01, 0.00000001, n, 1, 1, True);
    for i := 1 to n do
    begin
      dz[i] := z[1] * Power(com_ratio + c * i, i - 1);
      z[i] := z[i-1] + dz[i];
      z_c[i] := (z[i] + z[i-1]) / 2;
    //  z[n] := z_lb;
    //  dz[n] := z[n] - z[n-1];
    end;
  end;
  {
  dum := 0;
  for i := 1 to n do
    dum := dum + dz[i];
  ShowMessage(FloatToStr(dum));
  }
  z_c[n] := (z[n] + z[n-1]) / 2;
  z[n+1]  := z[n] + dz[n];
  dz[n+1] := dz[n];
  //-- layer identification --//
  for i := 1 to n - 1 do
  begin
    j := 1;
    while z[i] > z_border[j] + 0.000001 do
      j := j + 1;
    L[i] := j;
  end;
  //--------------------------//
  L[0] := L[1];
  L[n] := L[n-1];
  L[n+1] := L[n-1];
  dz[0] := dz[1];
  z[-1] := 0; //imaginary value
  //--- observation point ---//
  for j := 1 to NumObs do
  begin
    i := 0;
    while z_c[i] <= z_obs[j] do
      i := i + 1;
    NCtime[j] := i;  //NCtime :compartment number containing the point of continuous mesurment
  end;
  //---- albedo ----//
  i := 0;
  while (z[i] < Soil.ThickAlb[1]) and (i < n) do
    i := i + 1;
  NCalb := i
end;

function TSimulator.GetResidual(x: Double; n: Integer): Double;
var
  Sn: Double;
  i: Integer;
begin
  com_ratio  := Power(dz_bot / z[1], 1 / (n - 1)) - x * n;
  Sn := 0;
  for i := 1 to n do
    Sn := Sn + z[1] * Power(com_ratio + x * i, i - 1);
  Result := z_lb - Sn;
end;

//----------------------------------------------------------
procedure TSimulator.SolveTheCoefficientMatrix;
var i: Integer;
begin
  // solving symmetric and tridiagonal matrix using the Thomas algorithm
  for i := 1 to n - 1 do
  begin
    dum := Ath[i+1] / Bth[i];
    Bth[i+1] := Bth[i+1] - dum * Cth[i];
    Dth[i+1] := Dth[i+1] - dum * Dth[i];
  end;
  x[n] := Dth[n] / Bth[n];
  for i := n - 1 downto 1 do
    x[i] := (Dth[i] - Cth[i] * x[i+1]) / Bth[i];
end;
//----------------------------------------------------------
function TSimulator.GetOneStepDisp(i: Integer): Double;
var RN: Double;
begin
  RN := Random;
  if RN < Ym[i] / 2 * (Xmax[i] - mean_[i]) then
    Result := SIGN[i] * (Xmax[i] - Sqrt(2 * RN * (Xmax[i] - mean_[i]) / Ym[i]))
  else
    begin
    dum := Sqr(Y0[i]) - 2 * (Ym[i] - Y0[i]) / mean_[i] * (RN - 1);
    if dum < 0.0001 then
      ShowMessage('!');
    Result := SIGN[i] * mean_[i] * (-Y0[i] + Sqrt(dum)) / (Ym[i] - Y0[i]);
    end;
end;
//----------------------------------------------------------
function TSimulator.GetMilliTime:Double;
var
  Present: TDateTime;
  hour,min,Sec, MSec: Word;
begin
  Present :=  Now;
  DecodeTime(Present,hour,min,sec,msec);
  GetMilliTime := hour*3600+min*60+sec+msec/1000
end;
//----------------------------------------------------------
function TSimulator.SetLengthRJ(dumstr: String; dumint: Shortint): String;
var i: Shortint;
begin
  for i := 1 to dumint - Length(dumstr) do
    dumstr := ' '+ dumstr;
  Result := dumstr;
end;
//----------------------------------------------------------

procedure TSimulator.SetScanningCurve(j, i: Integer; Drying_: Boolean);
var
  Wmd, Wmw : Double;
begin
  if Drying_ then
  begin
    case Soil.RF[j] of
    1:if P[i] < 0 then
        Wsat_A[i] := FindRoot(Soil.zeta[j], Soil.Wsat[j], 0.000001, i, j, 2, True)
      else
        Wsat_A[i] := Soil.Wsat[j];
    2:if P[i] < 0 then
      begin
        Wmd := Soil.Water_Content(P[i], T[i], Soil.Wsat[j], Soil.zeta[j], Soil.Wr[j], j, True);
        Wsat_A[i] := Soil.Wr[j] + (W[i] - Soil.Wr[j]) / (Wmd - Soil.Wr[j]) * (Soil.Wsat[j] - Soil.Wr[j])
      end;
      else
        Wsat_A[i] := Soil.Wsat[j];
    end;
    zeta_A[i] := Soil.zeta[j];
    Wr_A[i]   := Soil.Wr[j];
  end
  else
  begin
    case Soil.RF[j] of
    1:if P[i] < 0 then
        zeta_A[i] := FindRoot(0.0001, Soil.Wsat[j] - 0.0001, 0.000001, i, j, 2, False)
      else
        zeta_A[i] := Soil.zeta[j];
    2:if P[i] < 0 then
      begin
        Wmw := Soil.Water_Content(P[i], T[i], Soil.Wsat[j], Soil.zeta[j], Soil.Wr[j], j, False);
        Wr_A[i] := Soil.Wsat[j] - (Soil.Wsat[j] - W[i]) / (Soil.Wsat[j] - Wmw) * (Soil.Wsat[j] - Soil.Wr[j])
      end;
      else
        Wr_A[i] := Soil.Wr[j];
    end;
    Wsat_A[i] := Soil.Wsat[j];
  end;
end;

function TSimulator.GetScaledParameter(x: Double; i, j: Integer; Drying_: Boolean): Double;
begin
  if Drying_ then
    Result := W[i] - Soil.Water_Content(P[i], T[i], x, Soil.zeta[j], Soil.Wr[j], j, True)
  else
    Result := W[i] - Soil.Water_Content(P[i], T[i], Soil.Wsat[j], x, Soil.Wr[j], j, False);
end;
end.
